# encoding: utf-8

import time


class SimpleTimer(object):
    """All time are counted relative to start."""
    def __init__(self):
        self.start = 0
        self.events = []
    
    def tick(self, event_name="Untitled"):
        now = time.time()
        if not self.start:
            self.start = now
            last = now
        else:
            _, last = self.events[-1]
        total = now - self.start
        delta = now - last
        
        self.events.append((event_name, now))
        
        print "Total {:3.1f}    | Delta {:3.1f}    | {:s}".format(total, delta, event_name)


if __name__ == "__main__":
    timer = SimpleTimer()
    timer.tick('start')
    time.sleep(3)
    timer.tick('e1')
    time.sleep(2)
    timer.tick('e2')



# -*- encoding: utf-8 -*-

"""
1. filter universe: separate helper functions
2. calc weights
3. generate trades

------------------------

- modify models: register function (with context parameter)
- modify AlphaStrategy: inheritate

------------------------

suspensions and limit reachers:
1. deal with them in re_balance function, not in filter_universe
2. do not care about them when construct portfolio
3. subtract market value and re-normalize weights (positions) after (daily) market open, before sending orders
"""
import time

from jaqs.data import RemoteDataService
from jaqs.trade import AlphaStrategy

import jaqs.trade.analyze as ana
from jaqs.trade import AlphaBacktestInstance
from jaqs.trade import PortfolioManager
from jaqs.trade import AlphaTradeApi
from jaqs.trade import model
from jaqs.data import DataView
import jaqs.util as jutil

from config_path import DATA_CONFIG_PATH
data_config = jutil.read_json(DATA_CONFIG_PATH)

dataview_dir_path = '../../output/prepared/test_backtest'
backtest_result_dir_path = '../../output/test_backtest'


def save_dataview():
    ds = RemoteDataService()
    ds.init_from_config(data_config)
    
    dv = DataView()
    
    props = {'start_date': 20170101, 'end_date': 20171030, 'universe': '000300.SH',
             'fields': ('open,high,low,close,vwap,volume,turnover,sw1,'
                        # + 'pb,net_assets,'
                        + 'eps_basic,total_mv,tot_profit,int_income'
                        ),
             'freq': 1}
    
    dv.init_from_config(props, ds)
    dv.prepare_data()
    
    factor_formula = 'close >= Delay(Ts_Max(close, 20), 1)'  # 20 days new high
    factor_name = 'new_high'
    dv.add_formula(factor_name, factor_formula, is_quarterly=False)
    
    dv.add_formula('total_profit_growth', formula='Return(tot_profit, 4)', is_quarterly=True)
    
    dv.save_dataview(folder_path=dataview_dir_path)


def my_selector(context, user_options=None):
    growth_rate = context.snapshot['total_profit_growth']
    return growth_rate > 0.05


def my_selector_no_new_stocks(context, user_options=None):
    import pandas as pd
    td = context.trade_date
    
    df_inst = context.dataview.data_inst
    ser_list_date = pd.to_datetime(df_inst['list_date'], format="%Y%m%d")
    td_dt = pd.to_datetime(td, format="%Y%m%d")
    diff = (td_dt - ser_list_date).dt.days
    mask = diff > 50
    return mask


def my_factor(context, user_options=None):
    res = context.snapshot_sub['new_high']
    return res


def my_commission(symbol, turnover, context=None, user_options=None):
    return turnover * user_options['myrate']


def test_alpha_strategy_dataview():
    save_dataview()
    
    dv = DataView()
    dv.load_dataview(folder_path=dataview_dir_path)
    
    props = {
        "start_date": dv.start_date,
        "end_date": dv.end_date,
    
        "period": "week",
        "days_delay": 0,
    
        "init_balance": 1e8,
        "position_ratio": 0.7,
        'commission_rate': 0.0
        }

    trade_api = AlphaTradeApi()
    bt = AlphaBacktestInstance()
    
    risk_model = model.FactorRiskModel()
    signal_model = model.FactorSignalModel()
    cost_model = model.SimpleCostModel()
    stock_selector = model.StockSelector()
    
    signal_model.add_signal(name='my_factor', func=my_factor)
    cost_model.consider_cost(name='my_commission', func=my_commission, options={'myrate': 1e-2})
    stock_selector.add_filter(name='total_profit_growth', func=my_selector)
    stock_selector.add_filter(name='no_new_stocks', func=my_selector_no_new_stocks)
    
    strategy = AlphaStrategy(signal_model=signal_model, stock_selector=stock_selector,
                             cost_model=cost_model, risk_model=risk_model,
                             pc_method='factor_value_weight')
    pm = PortfolioManager()
    # strategy = AlphaStrategy(signal_model=signal_model, pc_method='factor_value_weight')
    # strategy = AlphaStrategy(stock_selector=stock_selector, pc_method='market_value_weight')
    # strategy = AlphaStrategy()

    context = model.AlphaContext(dataview=dv, trade_api=trade_api,
                                 instance=bt, strategy=strategy, pm=pm)
    for mdl in [risk_model, signal_model, cost_model, stock_selector]:
        mdl.register_context(context)

    bt.init_from_config(props)

    bt.run_alpha()
    
    bt.save_results(folder_path=backtest_result_dir_path)


def test_backtest_analyze():
    ta = ana.AlphaAnalyzer()
    
    dv = DataView()
    dv.load_dataview(folder_path=dataview_dir_path)
    
    ta.initialize(dataview=dv, file_folder=backtest_result_dir_path)

    print "process trades..."
    ta.process_trades()
    print "get daily stats..."
    ta.get_daily()
    print "calc strategy return..."
    ta.get_returns(consider_commission=True)
    # position change info is huge!
    # print "get position change..."
    # ta.get_pos_change_info()
    
    selected_sec = list(ta.universe)[:2]
    if len(selected_sec) > 0:
        print "Plot single securities PnL"
        for symbol in selected_sec:
            df_daily = ta.daily.get(symbol, None)
            if df_daily is not None:
                ana.plot_trades(df_daily, symbol=symbol, save_folder=backtest_result_dir_path)
    
    print "Plot strategy PnL..."
    ta.plot_pnl(backtest_result_dir_path)
    
    print "generate report..."
    static_folder = jutil.join_relative_path("trade/analyze/static")
    ta.gen_report(source_dir=static_folder, template_fn='report_template.html',
                  out_folder=backtest_result_dir_path,
                  selected=selected_sec)
    
    ta.brinson('sw1')


if __name__ == "__main__":
    t_start = time.time()
    
    test_alpha_strategy_dataview()
    test_backtest_analyze()
    
    t3 = time.time() - t_start
    print "\n\n\nTime lapsed in total: {:.1f}".format(t3)




# encoding: UTF-8


class Instrument(object):
    def __init__(self):
        self.symbol = ""
        self.multiplier = 0.0
        self.inst_type = 0
        self.market = ""
        self.list_date = 0
        self.delist_date = 99999999
    
    def is_stock(self):
        return self.inst_type == 1
    
    def is_future(self):
        res = (self.inst_type == 101
               or self.inst_type == 102
               or self.inst_type == 103)
        return res


class InstManager(object):
    def __init__(self, data_api, inst_type="", symbol=""):
        self.data_api = data_api
        
        self.inst_map = {}
        self.load_instruments(inst_type=inst_type, symbol=symbol)
    
    def load_instruments(self, inst_type="", symbol=""):
        fields = ['symbol', 'inst_type', 'market', 'status', 'multiplier', 'list_date', 'delist_date']
        res = self.data_api.query_inst_info(symbol=symbol, fields=','.join(fields), inst_type=inst_type)
        res = res.reset_index()

        dic_of_dic = res.to_dict(orient='index')
        res = {v['symbol']: {v_key: v_value for v_key, v_value in v.viewitems() if v_key in fields}
               for _, v in dic_of_dic.viewitems()}
        for k, v in res.viewitems():
            inst = Instrument()
            inst.__dict__.update(v)
            self.inst_map[k] = inst
        print
    
    def get_instrument(self, code):
        return self.inst_map.get(code, None)



# encoding: utf-8

from jaqs.trade.event import EventEngine, Event, EVENT_TYPE
from jaqs.data.basic import Quote
import jaqs.util as jutil


class EventRealTimeInstance(EventEngine):
    """
    Attributes
    ----------
    start_date : int
    end_date : int
    
    """
    def __init__(self):
        super(EventRealTimeInstance, self).__init__()
        
        self.start_date = 0
        self.end_date = 0
        
        self.props = None
        
        self.ctx = None
    
    def init_from_config(self, props):
        """
        
        Parameters
        ----------
        props : dict

        """
        for name in ['start_date', 'end_date']:
            if name not in props:
                pass
                # raise ValueError("{} must be provided in props.".format(name))
        
        self.props = props
        self.start_date = props.get("start_date")
        self.end_date = props.get("end_date")

        for obj in ['data_api', 'trade_api', 'pm', 'strategy']:
            if hasattr(self.ctx, obj):
                getattr(self.ctx, obj).init_from_config(props)

    def register_context(self, context=None):
        self.ctx = context

    # -------------------------------------------------------------------------------------------
    # Run
    def run(self):
        """
        Listen to certain events and run the EventEngine.
        Events include:
            1. market_data are from DataService
            2. trades & orders indications are from TradeApi.
            3. etc.

        """

        self.register(EVENT_TYPE.MARKET_DATA, self.on_quote)
        
        self.register(EVENT_TYPE.TASK_STATUS_IND, self.on_task_status)
        self.register(EVENT_TYPE.ORDER_RSP, self.on_order_rsp)
        self.register(EVENT_TYPE.TASK_RSP, self.on_task_rsp)
        self.register(EVENT_TYPE.TRADE_IND, self.on_trade)
        self.register(EVENT_TYPE.ORDER_STATUS_IND, self.on_order_status)
        
        self.start(timer=False)
    
    def on_quote(self, event):
        quote_dic = event.dic['quote']
        quote = Quote.create_from_dict(quote_dic)
        self.ctx.strategy.on_tick(quote)
    
    def on_order_rsp(self, event):
        rsp = event.dic['rsp']
        self.ctx.strategy.on_order_rsp(rsp)

    def on_task_rsp(self, event):
        rsp = event.dic['rsp']
        self.ctx.strategy.on_task_rsp(rsp)
    
    def on_trade(self, event):
        ind = event.dic['ind']
        self.ctx.strategy.on_trade(ind)

    def on_order_status(self, event):
        ind = event.dic['ind']
        self.ctx.strategy.on_order_status(ind)

    def on_task_status(self, event):
        ind = event.dic['ind']
        self.ctx.strategy.on_task_status(ind)
    
    # ---------------------------------------------------------
    # Save Results
    def save_results(self, folder_path='.'):
        import os
        import pandas as pd
        folder_path = os.path.abspath(folder_path)
    
        trades = self.ctx.pm.trades
    
        type_map = {'task_id': str,
                    'entrust_no': str,
                    'entrust_action': str,
                    'symbol': str,
                    'fill_price': float,
                    'fill_size': float,
                    'fill_date': int,
                    'fill_time': int,
                    'fill_no': str,
                    'commission': float}
        # keys = trades[0].__dict__.keys()
        ser_list = dict()
        for key in type_map.keys():
            v = [t.__getattribute__(key) for t in trades]
            ser = pd.Series(data=v, index=None, dtype=type_map[key], name=key)
            ser_list[key] = ser
        df_trades = pd.DataFrame(ser_list)
        df_trades.index.name = 'index'
    
        trades_fn = os.path.join(folder_path, 'trades.csv')
        configs_fn = os.path.join(folder_path, 'configs.json')
        jutil.create_dir(trades_fn)
    
        df_trades.to_csv(trades_fn)
        jutil.save_json(self.props, configs_fn)
    
        print ("Backtest results has been successfully saved to:\n" + folder_path)



# encoding: UTF-8

from abc import abstractmethod

import numpy as np
import pandas as pd

from jaqs.trade.event import EVENT_TYPE, Event
from jaqs.data import DataApi
from jaqs.data import align
import jaqs.util as jutil


class DataService(object):
    """
    Abstract base class providing both historic and live data
    from various data sources.
    Current API version: 1.8

    Derived classes of DataServer hide different data source, but use the same API.

    Attributes
    ----------
    source_name : str
        Name of data source.

    Methods
    -------
    subscribe
    quote
    daily
    bar
    tick
    query

    """
    def __init__(self, name=""):
        
        if name:
            self.source_name = name
        else:
            self.source_name = str(self.__class__.__name__)
        
        self.ctx = None
    
    '''
    def subscribe(self, targets, callback):
        """
        Subscribe real time market data, including bar and tick,
        processed by respective callback function.

        Parameters
        ----------
        targets : str
            Security and type, eg. "000001.SH/tick,cu1709.SHF/1m"
        callback : dict of {str: callable}
            {'on_tick': func1, 'on_bar': func2}
            Call back functions.

        """
        # TODO for now it will not publish event
        for target in targets.split(','):
            sec, data_type = target.split('/')
            if data_type == 'tick':
                func = callback['on_tick']
            else:
                func = callback['on_bar']
            self.add_subscriber(func, target)
    
    '''
    def register_context(self, context):
        self.ctx = context
    
    @abstractmethod
    def quote(self, symbol, fields=""):
        """
        Query latest market data in DataFrame.
        
        Parameters
        ----------
        symbol : str
        fields : str, optional
            default ""

        Returns
        -------
        df : pd.DataFrame
        msg : str
            error code and error message joined by comma

        """
        pass
    
    @abstractmethod
    def daily(self, symbol, start_date, end_date, fields="", adjust_mode=None):
        """
        Query dar bar,
        support auto-fill suspended securities data,
        support auto-adjust for splits, dividends and distributions.

        Parameters
        ----------
        symbol : str
            support multiple securities, separated by comma.
        start_date : int or str
            YYYMMDD or 'YYYY-MM-DD'
        end_date : int or str
            YYYMMDD or 'YYYY-MM-DD'
        fields : str, optional
            separated by comma ',', default "" (all fields included).
        adjust_mode : str or None, optional
            None for no adjust;
            'pre' for forward adjust;
            'post' for backward adjust.

        Returns
        -------
        df : pd.DataFrame
            columns:
                symbol, code, trade_date, open, high, low, close, volume, turnover, vwap, oi, suspended
        msg : str
            error code and error message joined by comma

        Examples
        --------
        df, msg = api.daily("00001.SH,cu1709.SHF",start_date=20170503, end_date=20170708,
                            fields="open,high,low,last,volume", fq=None, skip_suspended=True)

        """
        pass
    
    @abstractmethod
    def bar(self, symbol, start_time=200000, end_time=160000, trade_date=None, freq='1m', fields=""):
        """
        Query minute bars of various type, return DataFrame.

        Parameters
        ----------
        symbol : str
            support multiple securities, separated by comma.
        start_time : int (HHMMSS) or str ('HH:MM:SS')
            Default is market open time.
        end_time : int (HHMMSS) or str ('HH:MM:SS')
            Default is market close time.
        trade_date : int (YYYMMDD) or str ('YYYY-MM-DD')
            Default is current trade_date.
        fields : str, optional
            separated by comma ',', default "" (all fields included).
        freq : trade.common.MINBAR_TYPE, optional
            {'1m', '5m', '15m'}, Minute bar type, default is '1m'

        Returns
        -------
        df : pd.DataFrame
            columns:
                symbol, code, date, time, trade_date, freq, open, high, low, close, volume, turnover, vwap, oi
        msg : str
            error code and error message joined by comma

        Examples
        --------
        df, msg = api.bar("000001.SH,cu1709.SHF", start_time="09:56:00", end_time="13:56:00",
                          trade_date="20170823", fields="open,high,low,last,volume", freq="5m")

        """
        # TODO data_server DOES NOT know "current date".
        pass
    
    @abstractmethod
    def tick(self, symbol, start_time=200000, end_time=160000, trade_date=None, fields=""):
        """
        Query tick data in DataFrame.
        
        Parameters
        ----------
        symbol : str
        start_time : int (HHMMSS) or str ('HH:MM:SS')
            Default is market open time.
        end_time : int (HHMMSS) or str ('HH:MM:SS')
            Default is market close time.
        trade_date : int (YYYMMDD) or str ('YYYY-MM-DD')
            Default is current trade_date.
        fields : str, optional
            separated by comma ',', default "" (all fields included).

        Returns
        -------
        df : pd.DataFrame
        err_msg : str
            error code and error message joined by comma
            
        """
        pass
    
    @abstractmethod
    def query(self, view, filter, fields):
        """
        Query reference data.
        Input query type and parameters, return DataFrame.
        
        Parameters
        ----------
        view : str
            Type of reference data. See doc for details.
        filter : str
            Query conditions, separated by '&'.
        fields : str
            Fields to return, separated by ','.

        Returns
        -------
        df : pd.DataFrame
        err_msg : str
            error code and error message joined by comma

        """
        pass
    
    @abstractmethod
    def get_split_dividend(self):
        pass
    
    def get_suspensions(self):
        pass


class Singleton(type):
    _instances = {}
    
    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]


class RemoteDataService(DataService):
    """
    RemoteDataService is a concrete class using data from remote server's database.

    """
    __metaclass__ = Singleton
    # TODO no validity check for input parameters
    
    def __init__(self):
        DataService.__init__(self)
        
        self.data_api = None
        
        self.REPORT_DATE_FIELD_NAME = 'report_date'
        # self.calendar = None
        
    def __del__(self):
        self.data_api.close()

    def init_from_config(self, props):
        # do not initialize and login again
        if self.data_api is not None and self.data_api._loggined:
            return
        
        if props is None:
            props = dict()
            
        if self.data_api is not None:
            if len(props) == 0:
                return
            else:
                self.data_api.close()
            
        def get_from_list_of_dict(l, key, default=None):
            res = None
            for dic in l:
                res = dic.get(key, None)
                if res is not None:
                    break
            if res is None:
                res = default
            return res
        
        props_default = dict()  # jutil.read_json(jutil.join_relative_path('etc/data_config.json'))
        dic_list = [props, props_default]
        
        address = get_from_list_of_dict(dic_list, "remote.data.address", "")
        username = get_from_list_of_dict(dic_list, "remote.data.username", "")
        password = get_from_list_of_dict(dic_list, "remote.data.password", "")
        if address is None or username is None or password is None:
            raise ValueError("no address, username or password available!")
        time_out = get_from_list_of_dict(dic_list, "timeout", 60)

        self.data_api = DataApi(address, use_jrpc=False)
        self.data_api.set_timeout(timeout=time_out)
        print("\nDataApi login: {}@{}".format(username, address))
        r, msg = self.data_api.login(username=username, password=password)
        if not r:
            print("    login failed: msg = '{}'\n".format(msg))
        else:
            print "    login success \n"
        
        # self.calendar = Calendar(self)

    # -----------------------------------------------------------------------------------
    # Basic APIs
    def daily(self, symbol, start_date, end_date,
              fields="", adjust_mode=None):
        df, err_msg = self.data_api.daily(symbol=symbol, start_date=start_date, end_date=end_date,
                                          fields=fields, adjust_mode=adjust_mode, data_format="")
        # trade_status performance warning
        # TODO there will be duplicate entries when on stocks' IPO day
        df = df.drop_duplicates()
        return df, err_msg

    def bar(self, symbol,
            start_time=200000, end_time=160000, trade_date=None,
            freq='1M', fields=""):
        df, msg = self.data_api.bar(symbol=symbol, fields=fields,
                                    start_time=start_time, end_time=end_time, trade_date=trade_date,
                                    freq='1M', data_format="")
        return df, msg
    
    def query(self, view, filter="", fields="", **kwargs):
        """
        Get various reference data.
        
        Parameters
        ----------
        view : str
            data source.
        fields : str
            Separated by ','
        filter : str
            filter expressions.
        kwargs

        Returns
        -------
        df : pd.DataFrame
        msg : str
            error code and error message, joined by ','
        
        Examples
        --------
        res3, msg3 = ds.query("lb.secDailyIndicator", fields="price_level,high_52w_adj,low_52w_adj",\
                              filter="start_date=20170907&end_date=20170907",\
                              orderby="trade_date",\
                              data_format='pandas')
            view does not change. fileds can be any field predefined in reference data api.

        """
        df, msg = self.data_api.query(view, fields=fields, filter=filter, data_format="", **kwargs)
        return df, msg

    # -----------------------------------------------------------------------------------
    # Convenient Functions
    def get_trade_date_range_OLD(self, start_date, end_date):
        return self.calendar.get_trade_date_range(start_date, end_date)
    
    @staticmethod
    def _dic2url(d):
        """
        Convert a dict to str like 'k1=v1&k2=v2'
        
        Parameters
        ----------
        d : dict

        Returns
        -------
        str

        """
        l = ['='.join([key, str(value)]) for key, value in d.viewitems()]
        return '&'.join(l)

    def query_lb_fin_stat(self, type_, symbol, start_date, end_date, fields="", drop_dup_cols=None):
        """
        Helper function to call data_api.query with 'lb.income' more conveniently.
        
        Parameters
        ----------
        type_ : {'income', 'balance_sheet', 'cash_flow'}
        symbol : str
            separated by ','
        start_date : int
            Annoucement date in results will be no earlier than start_date
        end_date : int
            Annoucement date in results will be no later than start_date
        fields : str, optional
            separated by ',', default ""
        drop_dup_cols : list or tuple
            Whether drop duplicate entries according to drop_dup_cols.

        Returns
        -------
        df : pd.DataFrame
            index date, columns fields
        msg : str

        """
        view_map = {'income': 'lb.income', 'cash_flow': 'lb.cashFlow', 'balance_sheet': 'lb.balanceSheet',
                    'fin_indicator': 'lb.finIndicator'}
        view_name = view_map.get(type_, None)
        if view_name is None:
            raise NotImplementedError("type_ = {:s}".format(type_))
        
        dic_argument = {'symbol': symbol,
                        'start_date': start_date,
                        'end_date': end_date,
                        # 'update_flag': '0'
                       }
        if view_name != 'lb.finIndicator':
            dic_argument.update({'report_type': '408001000'})  # we do not use single quarter single there are zeros
            """
            408001000: joint
            408002000: joint (single quarter)
            """
        
        filter_argument = self._dic2url(dic_argument)  # 0 means first time, not update
        
        res, msg = self.query(view_name, fields=fields, filter=filter_argument,
                              order_by=self.REPORT_DATE_FIELD_NAME)
        
        # change data type
        try:
            cols = list(set.intersection({'ann_date', 'report_date'}, set(res.columns)))
            dic_dtype = {col: int for col in cols}
            res = res.astype(dtype=dic_dtype)
        except:
            pass
        
        if drop_dup_cols is not None:
            res = res.sort_values(by=drop_dup_cols, axis=0)
            res = res.drop_duplicates(subset=drop_dup_cols, keep='first')
        
        return res, msg

    def query_lb_dailyindicator(self, symbol, start_date, end_date, fields=""):
        """
        Helper function to call data_api.query with 'lb.secDailyIndicator' more conveniently.
        
        Parameters
        ----------
        symbol : str
            separated by ','
        start_date : int
        end_date : int
        fields : str, optional
            separated by ',', default ""

        Returns
        -------
        df : pd.DataFrame
            index date, columns fields
        msg : str
        
        """
        filter_argument = self._dic2url({'symbol': symbol,
                                         'start_date': start_date,
                                         'end_date': end_date})
    
        return self.query("lb.secDailyIndicator",
                          fields=fields,
                          filter=filter_argument,
                          orderby="trade_date")

    def get_index_weights(self, index, trade_date):
        """
        Return all securities that have been in index during start_date and end_date.
        
        Parameters
        ----------
        index : str
            separated by ','
        trade_date : int

        Returns
        -------
        pd.DataFrame

        """
        if index == '000300.SH':
            index = '399300.SZ'
            
        filter_argument = self._dic2url({'index_code': index,
                                         'trade_date': trade_date})
    
        df_io, msg = self.query("lb.indexWeight", fields="",
                                filter=filter_argument)
        if msg != '0,':
            print msg
        df_io = df_io.set_index('symbol')
        df_io = df_io.astype({'weight': float, 'trade_date': int})
        df_io.loc[:, 'weight'] = df_io['weight'] / 100.
        return df_io

    def get_index_weights_daily(self, index, start_date, end_date):
        """
        Return all securities that have been in index during start_date and end_date.
        
        Parameters
        ----------
        index : str
        start_date : int
        end_date : int

        Returns
        -------
        res : pd.DataFrame
            Index is trade_date, columns are symbols.

        """
        # TODO: temparary api
        trade_dates = self.get_trade_date_range(start_date, end_date)
        start_date, end_date = trade_dates[0], trade_dates[-1]
        td = start_date
        
        dic = dict()
        symbols_set = set()
        while True:
            if td > end_date:
                break
            df = self.get_index_weights(index, td)
            # update_date = df['trade_date'].iat[0]
            # if update_date >= start_date and update_date <= end_date:
            symbols_set.update(set(df.index))
            dic[td] = df['weight']
            
            td = jutil.get_next_period_day(td, 'month', 1)
        merge = pd.concat(dic, axis=1).T
        merge = merge.fillna(0.0)  # for those which are not components
        res = pd.DataFrame(index=trade_dates, columns=sorted(list(symbols_set)), data=np.nan)
        res.update(merge)
        res = res.fillna(method='ffill')
        res = res.loc[start_date: end_date]
        return res
    
    def _get_index_comp(self, index, start_date, end_date):
        """
        Return all securities that have been in index during start_date and end_date.
        
        Parameters
        ----------
        index : str
            separated by ','
        start_date : int
        end_date : int

        Returns
        -------
        list

        """
        filter_argument = self._dic2url({'index_code': index,
                                         'start_date': start_date,
                                         'end_date': end_date})
    
        df_io, msg = self.query("lb.indexCons", fields="",
                                filter=filter_argument, orderby="symbol")
        return df_io, msg
    
    def get_index_comp(self, index, start_date, end_date):
        """
        Return list of symbols that have been in index during start_date and end_date.
        
        Parameters
        ----------
        index : str
            separated by ','
        start_date : int
        end_date : int

        Returns
        -------
        list

        """
        df_io, msg = self._get_index_comp(index, start_date, end_date)
        if msg != '0,':
            print msg
        return list(np.unique(df_io.loc[:, 'symbol']))
    
    def get_index_comp_df(self, index, start_date, end_date):
        """
        Get index components on each day during start_date and end_date.
        
        Parameters
        ----------
        index : str
            separated by ','
        start_date : int
        end_date : int

        Returns
        -------
        res : pd.DataFrame
            index dates, columns all securities that have ever been components,
            values are 0 (not in) or 1 (in)

        """
        df_io, msg = self._get_index_comp(index, start_date, end_date)
        if msg != '0,':
            print msg
        
        def str2int(s):
            if isinstance(s, (str, unicode)):
                return int(s) if s else 99999999
            elif isinstance(s, (int, np.integer, float, np.float)):
                return s
            else:
                raise NotImplementedError("type s = {}".format(type(s)))
        df_io.loc[:, 'in_date'] = df_io.loc[:, 'in_date'].apply(str2int)
        df_io.loc[:, 'out_date'] = df_io.loc[:, 'out_date'].apply(str2int)
        
        # df_io.set_index('symbol', inplace=True)
        dates = self.get_trade_date_range(start_date=start_date, end_date=end_date)

        dic = dict()
        gp = df_io.groupby(by='symbol')
        for sec, df in gp:
            mask = np.zeros_like(dates, dtype=int)
            for idx, row in df.iterrows():
                bool_index = np.logical_and(dates > row['in_date'], dates < row['out_date'])
                mask[bool_index] = 1
            dic[sec] = mask
            
        res = pd.DataFrame(index=dates, data=dic)
        
        return res

    def get_industry_daily(self, symbol, start_date, end_date, type_='SW', level=1):
        """
        Get index components on each day during start_date and end_date.
        
        Parameters
        ----------
        symbol : str
            separated by ','
        start_date : int
        end_date : int
        type_ : {'SW', 'ZZ'}

        Returns
        -------
        res : pd.DataFrame
            index dates, columns symbols
            values are industry code

        """
        df_raw = self.get_industry_raw(symbol, type_=type_, level=level)
        
        dic_sec = jutil.group_df_to_dict(df_raw, by='symbol')
        dic_sec = {sec: df.sort_values(by='in_date', axis=0).reset_index()
                   for sec, df in dic_sec.viewitems()}

        df_ann_tmp = pd.concat({sec: df.loc[:, 'in_date'] for sec, df in dic_sec.viewitems()}, axis=1)
        df_value_tmp = pd.concat({sec: df.loc[:, 'industry{:d}_code'.format(level)]
                                  for sec, df in dic_sec.viewitems()},
                                 axis=1)
        
        idx = np.unique(np.concatenate([df.index.values for df in dic_sec.values()]))
        symbol_arr = np.sort(symbol.split(','))
        df_ann = pd.DataFrame(index=idx, columns=symbol_arr, data=np.nan)
        df_ann.loc[df_ann_tmp.index, df_ann_tmp.columns] = df_ann_tmp
        df_value = pd.DataFrame(index=idx, columns=symbol_arr, data=np.nan)
        df_value.loc[df_value_tmp.index, df_value_tmp.columns] = df_value_tmp

        dates_arr = self.get_trade_date_range(start_date, end_date)
        df_industry = align.align(df_value, df_ann, dates_arr)
        
        # TODO before industry classification is available, we assume they belong to their first group.
        df_industry = df_industry.fillna(method='bfill')
        df_industry = df_industry.astype(str)
        
        return df_industry
        
    def get_industry_raw(self, symbol, type_='ZZ', level=1):
        """
        Get daily industry of securities from ShenWanZhiShu or ZhongZhengZhiShu.
        
        Parameters
        ----------
        symbol : str
            separated by ','
        type_ : {'SW', 'ZZ'}
        level : {1, 2, 3, 4}
            Use which level of industry index classification.

        Returns
        -------
        df : pd.DataFrame

        """
        if type_ == 'SW':
            src = u'申万研究所'.encode('utf-8')
            if level not in [1, 2, 3, 4]:
                raise ValueError("For [SW], level must be one of {1, 2, 3, 4}")
        elif type_ == 'ZZ':
            src = u'中证指数有限公司'.encode('utf-8')
            if level not in [1, 2, 3, 4]:
                raise ValueError("For [ZZ], level must be one of {1, 2}")
        else:
            raise ValueError("type_ must be one of SW of ZZ")
        
        filter_argument = self._dic2url({'symbol': symbol,
                                         'industry_src': src})
        fields_list = ['symbol', 'industry{:d}_code'.format(level), 'industry{:d}_name'.format(level)]
    
        df_raw, msg = self.query("lb.secIndustry", fields=','.join(fields_list),
                                 filter=filter_argument, orderby="symbol")
        if msg != '0,':
            print msg
        
        df_raw = df_raw.astype(dtype={'in_date': int,
                                      # 'out_date': int
                                     })
        return df_raw.drop_duplicates()

    def get_adj_factor_daily(self, symbol, start_date, end_date, div=False):
        """
        Get index components on each day during start_date and end_date.
        
        Parameters
        ----------
        symbol : str
            separated by ','
        start_date : int
        end_date : int
        div : bool
            False for normal adjust factor, True for diff.

        Returns
        -------
        res : pd.DataFrame
            index dates, columns symbols
            values are industry code

        """
        df_raw = self.get_adj_factor_raw(symbol, start_date=start_date, end_date=end_date)
    
        dic_sec = jutil.group_df_to_dict(df_raw, by='symbol')
        dic_sec = {sec: df.set_index('trade_date').loc[:, 'adjust_factor']
                   for sec, df in dic_sec.viewitems()}
        
        # TODO: duplicate codes with dataview.py: line 512
        res = pd.concat(dic_sec, axis=1)  # TODO: fillna ?
        
        idx = np.unique(np.concatenate([df.index.values for df in dic_sec.values()]))
        symbol_arr = np.sort(symbol.split(','))
        res_final = pd.DataFrame(index=idx, columns=symbol_arr, data=np.nan)
        res_final.loc[res.index, res.columns] = res

        # align to every trade date
        s, e = df_raw.loc[:, 'trade_date'].min(), df_raw.loc[:, 'trade_date'].max()
        dates_arr = self.get_trade_date_range(s, e)
        if not len(dates_arr) == len(res_final.index):
            res_final = res_final.reindex(dates_arr)
            
            res_final = res_final.fillna(method='ffill').fillna(method='bfill')

        if div:
            res_final = res_final.div(res_final.shift(1, axis=0)).fillna(1.0)
            
        # res = res.loc[start_date: end_date, :]

        return res_final

    def get_adj_factor_raw(self, symbol, start_date=None, end_date=None):
        """
        Query adjust factor for symbols.
        
        Parameters
        ----------
        symbol : str
            separated by ','
        start_date : int
        end_date : int

        Returns
        -------
        df : pd.DataFrame

        """
        if start_date is None:
            start_date = ""
        if end_date is None:
            end_date = ""
        
        filter_argument = self._dic2url({'symbol': symbol,
                                         'start_date': start_date, 'end_date': end_date})
        fields_list = ['symbol', 'trade_date', 'adjust_factor']

        df_raw, msg = self.query("lb.secAdjFactor", fields=','.join(fields_list),
                                 filter=filter_argument, orderby="symbol")
        if msg != '0,':
            print msg
        df_raw = df_raw.astype(dtype={'symbol': str,
                                    'trade_date': int,
                                    'adjust_factor': float
                                    })
        return df_raw.drop_duplicates()
    
    def query_inst_info(self, symbol, inst_type="", fields=""):
        if inst_type == "":
            inst_type = "1,2,3,4,5,101,102,103,104"
        
        filter_argument = self._dic2url({'symbol': symbol,
                                         'inst_type': inst_type})
    
        df_raw, msg = self.query("jz.instrumentInfo", fields=fields,
                                 filter=filter_argument, orderby="symbol")
        if msg != '0,':
            print msg

        dtype_map = {'symbol': str, 'list_date': int, 'delist_date': int, 'inst_type': int}
        cols = set(df_raw.columns)
        dtype_map = {k: v for k, v in dtype_map.viewitems() if k in cols}
        
        df_raw = df_raw.astype(dtype=dtype_map)
        
        res = df_raw.set_index('symbol')
        return res
    
    # -----------------------------------------------------------------------------------
    # subscribe for real time trading
    def subscribe(self, symbols):
        """
        
        Parameters
        ----------
        symbols : str
            Separated by ,

        """
        self.data_api.subscribe(symbols, func=self.mkt_data_callback)

    def mkt_data_callback(self, key, quote):
        e = Event(EVENT_TYPE.MARKET_DATA)
        # print quote
        e.dic = {'quote': quote}
        self.ctx.instance.put(e)
    
    # ---------------------------------------------------------------------
    # Calendar
    
    def get_trade_date_range(self, start_date, end_date):
        """
        Get array of trade dates within given range.
        Return zero size array if no trade dates within range.
        
        Parameters
        ----------
        start_date : int
            YYmmdd
        end_date : int

        Returns
        -------
        trade_dates_arr : np.ndarray
            dtype = int

        """
        filter_argument = self._dic2url({'start_date': start_date,
                                         'end_date': end_date})
    
        df_raw, msg = self.data_api.query("jz.secTradeCal", fields="trade_date",
                                          filter=filter_argument, orderby="")
        if df_raw.empty:
            return np.array([], dtype=int)
    
        trade_dates_arr = df_raw['trade_date'].values.astype(int)
        return trade_dates_arr

    def get_last_trade_date(self, date):
        """
        
        Parameters
        ----------
        date : int

        Returns
        -------
        res : int

        """
        dt = jutil.convert_int_to_datetime(date)
        delta = pd.Timedelta(weeks=2)
        dt_old = dt - delta
        date_old = jutil.convert_datetime_to_int(dt_old)
    
        dates = self.get_trade_date_range(date_old, date)
        mask = dates < date
        res = dates[mask][-1]
    
        return res

    def is_trade_date(self, date):
        """
        Check whether date is a trade date.

        Parameters
        ----------
        date : int

        Returns
        -------
        bool

        """
        dates = self.get_trade_date_range(date, date)
        return len(dates) > 0

    def get_next_trade_date(self, date):
        """
        
        Parameters
        ----------
        date : int

        Returns
        -------
        res : int

        """
        dt = jutil.convert_int_to_datetime(date)
        delta = pd.Timedelta(weeks=2)
        dt_new = dt + delta
        date_new = jutil.convert_datetime_to_int(dt_new)
    
        dates = self.get_trade_date_range(date, date_new)
        mask = dates > date
        res = dates[mask][0]
    
        return res

        



'''
class Calendar_OLD(object):
    """
    A calendar for manage trade date.
    
    Attributes
    ----------
    data_api :

    """
    
    def __init__(self, data_api=None):
        if data_api is None:
            ds = RemoteDataService()
            ds.init_from_config()
            self.data_api = ds
        else:
            self.data_api = data_api
    
    @staticmethod
    def _dic2url(d):
        """
        Convert a dict to str like 'k1=v1&k2=v2'
        
        Parameters
        ----------
        d : dict

        Returns
        -------
        str

        """
        l = ['='.join([key, str(value)]) for key, value in d.viewitems()]
        return '&'.join(l)
    
    def get_trade_date_range(self, start_date, end_date):
        """
        Get array of trade dates within given range.
        Return zero size array if no trade dates within range.
        
        Parameters
        ----------
        start_date : int
            YYmmdd
        end_date : int

        Returns
        -------
        trade_dates_arr : np.ndarray
            dtype = int

        """
        filter_argument = self._dic2url({'start_date': start_date,
                                         'end_date': end_date})
        
        df_raw, msg = self.data_api.query("jz.secTradeCal", fields="trade_date",
                                          filter=filter_argument, orderby="")
        if df_raw.empty:
            return np.array([], dtype=int)
        
        trade_dates_arr = df_raw['trade_date'].values.astype(int)
        return trade_dates_arr
    
    def get_last_trade_date(self, date):
        """
        
        Parameters
        ----------
        date : int

        Returns
        -------
        res : int

        """
        dt = jutil.convert_int_to_datetime(date)
        delta = pd.Timedelta(weeks=2)
        dt_old = dt - delta
        date_old = jutil.convert_datetime_to_int(dt_old)
        
        dates = self.get_trade_date_range(date_old, date)
        mask = dates < date
        res = dates[mask][-1]
        
        return res
    
    def is_trade_date(self, date):
        """
        Check whether date is a trade date.

        Parameters
        ----------
        date : int

        Returns
        -------
        bool

        """
        dates = self.get_trade_date_range(date, date)
        return len(dates) > 0
    
    def get_next_trade_date(self, date):
        """
        
        Parameters
        ----------
        date : int

        Returns
        -------
        res : int

        """
        dt = jutil.convert_int_to_datetime(date)
        delta = pd.Timedelta(weeks=2)
        dt_new = dt + delta
        date_new = jutil.convert_datetime_to_int(dt_new)
        
        dates = self.get_trade_date_range(date, date_new)
        mask = dates > date
        res = dates[mask][0]
        
        return res


'''













import threading
import _jrpc
import traceback

def set_log_dir(log_path):
    _jrpc.set_log_dir(log_path)

class JRpcWrap :
    
    def __init__(self) :
        self._handle = 0
        self._callback = None
        
        
    def __del__(self):
        self.close()

    def connect(self, addr):
        """
        Connect to remote server using addr. Return True if success else False
        """
        
        #There might have some incoming events in _jrpc.connect. In order not to
        # lose them, the _my_callback should be passed to _jrpc.

        self._handle = _jrpc.connect(addr, self._my_callback)
        return True if self._handle else False

    def close(self):
        _jrpc.close(self._handle)
        self._handle = 0

    def _my_callback(self, event, data):
        if self._callback :
            self._callback(event, data)

    def set_callback(self, callback):
        """
        Notification callback, prototype (event, data)
        event: 
                ".sys.connected",
                ".sys.disconnected"
                and others
        """
        self._callback = callback

    def call(self, method, param, callback, timeout=6000):
        """
        Async call. When the client receives result or timeout, callback will be called.
        Callback(call_id: Int, method: String, result: Any, error: [code, msg])
            if result is None, means call faiure

        return call_id: Int, zero means error.
        """
        assert self._handle, "none handle"
        return _jrpc.call(self._handle, method, param, callback, timeout)

class JRpcClient:
    """
    Simulate behaviour of jsonrpc.JsonRpcClient so AdminApi,TradeApi can be simple.
    """
    def __init__(self):
        self._jrpc_client = JRpcWrap()
        self.on_rpc_callback = None
        self.on_disconnected = None
        self.on_connected    = None

        self._jrpc_client.set_callback(self._my_callback)

    def __del__(self):
        self.close()

    def connect(self, addr):
        """
        return True or False
        """
        return self._jrpc_client.connect(addr)

    def close(self):
        if self._jrpc_client :
            self._jrpc_client.close()
            self._jrpc_client = None

    def _my_callback(self, method, data):
        if   method == ".sys.connected" :
            if self.on_connected : self.on_connected()

        elif method == ".sys.disconnected" :
            if self.on_disconnected : self.on_disconnected()

        else:
             if self.on_rpc_callback : self.on_rpc_callback(method, data)


    def set_heartbeat_options(self, interval, timeout):
        # self._heartbeat_interval = interval
        # self._heartbeat_timeout = timeout
        pass

    def call(self, method, params, timeout = 6) :
        if not timeout :
            try :
                self._jrpc_client.call(method, params, lambda id,m,r,e: None, timeout*1000)
                return {'result': True}
            except Exception as e:
                traceback.print_exc()
                return {'error': {'error': -1, 'message' : str(e)}}
        else:
            ret = {}
            cv = threading.Condition()
            try:
                def _lambda(id, m, r, e):
                    cv.acquire()
                    ret['result'] = r
                    ret['error']  = e
                    cv.notify()
                    cv.release()

                call_id = self._jrpc_client.call(method, params, _lambda, timeout*1000)
                if not call_id:
                    # if call_id is None, an exception should have been raised.
                    return {'error': {'error': -1, 'message' : 'unknown error'} }

            except Exception as e:
                traceback.print_exc()
                ret = {'error': {'error': -1, 'message' : str(e)}}
                return ret

            try:
                cv.acquire()
                if not ret:
                    # cv.wait a little longer than jrpc.call
                    cv.wait(timeout + 0.5)

                if not ret:
                    ret['error'] = {'error': -1, 'message' : 'timeout'}

            except Exception as e:
                traceback.print_exc()
                ret = {'error': {'error': -1, 'message' : str(e)}}
                    
            finally:
                cv.release()

            return ret




# encoding: utf-8
import numpy as np


def quantilize_without_nan(mat, n_quantiles=5, axis=-1):
    mask = np.isnan(mat)
    
    rank = mat.argsort(axis=axis).argsort(axis=axis)  # int
    
    count = np.sum(~mask, axis=axis)  # int
    divisor = count * 1. / n_quantiles  # float
    shape = list(mat.shape)
    shape[axis] = 1
    divisor = divisor.reshape(*shape)
    
    res = np.floor(rank / divisor) + 1.0
    res[mask] = np.nan
    
    return res



"""
The analyze module is used to analyze backtest results.

It can read csv and json files into memory, calculate various trade performance metrics
and output visualization of the results.
"""

from .analyze import EventAnalyzer, AlphaAnalyzer


__all__ = ['EventAnalyzer', 'AlphaAnalyzer']


import pandas as pd
from collections import namedtuple

def _to_date(row):
    date = int(row['DATE'])
    return pd.datetime( year=date/10000, month = date/ 100 % 100, day = date%100)

def _to_datetime(row):
    date = int(row['DATE'])
    time = int(row['TIME']) / 1000
    return pd.datetime( year=date/10000, month=date/ 100 % 100, day = date%100,
                        hour=time/10000, minute=time/100%100, second =time%100)

def _to_dataframe(cloumset, index_func = None, index_column = None):
    df = pd.DataFrame(cloumset)
    if index_func:
        df.index = df.apply(index_func, axis = 1)
    elif index_column:
        df.index = df[index_column]
        del df.index.name

    return df

def _error_to_str(error):
    if error:
        if error.has_key('message'):
            return str(error['error']) + "," + error['message']
        else:
            return str(error['error']) + ","
    else:
        return ","


def to_obj(class_name, data):
    try:
        if isinstance(data, (list, tuple)):
            result = []
            for d in data:
                result.append( namedtuple(class_name, d.keys())(*d.values()) )
            return result

        elif type(data) == dict :
            result = namedtuple(class_name, data.keys())(*data.values())
            return result
        else:
            return data
    except Exception, e:
        print class_name, data, e
        return data


def extract_result(cr, format="", index_column=None, class_name=""):
    """
        format supports pandas, obj.
    """
    
    err = _error_to_str(cr['error']) if cr.has_key('error') else None
    if cr.has_key('result'):
        if format == "pandas":
            if index_column :
                return (_to_dataframe(cr['result'], None, index_column), err)
            if 'TIME' in cr['result']:
                return (_to_dataframe(cr['result'], _to_datetime), err)
            elif 'DATE' in cr['result']:
                return (_to_dataframe(cr['result'], _to_date), err)
            else:
                return (_to_dataframe(cr['result']), err)

        elif format == "obj" and cr['result'] and class_name:
            r = cr['result']
            if isinstance(r, (list, tuple)):
                result = []
                for d in r:
                    result.append( namedtuple(class_name, d.keys())(*d.values()) )
            elif isinstance(r, dict):
                result = namedtuple(class_name, r.keys())(*r.values())
            else:
                result = r

            return (result, err)
        else:            
            return (cr['result'], err)
    else:
        return (None, err)



# encoding: UTF-8

import pandas as pd
import numpy as np
try:
    import pytest
except ImportError as e:
    if __name__ == "__main__":
        pass
    else:
        raise e
from jaqs.data import RemoteDataService
from jaqs.data import Parser
import jaqs.util as jutil

from config_path import DATA_CONFIG_PATH
data_config = jutil.read_json(DATA_CONFIG_PATH)


def test_group_rank():
    shape = (500, 3000)
    df_val = pd.DataFrame(np.random.rand(*shape))
    df_group = pd.DataFrame(np.random.randint(1, 5, size=shape[0] * shape[1]).reshape(*shape))
    expr = parser.parse('GroupRank(val, mygroup)')
    res = parser.evaluate({'val': df_val, 'mygroup': df_group})


def test_group_quantile():
    shape = (500, 3000)
    df_val = pd.DataFrame(np.random.rand(*shape))
    df_group = pd.DataFrame(np.random.randint(1, 5, size=shape[0] * shape[1]).reshape(*shape))
    expr = parser.parse('GroupQuantile(val, mygroup, 23)')
    res = parser.evaluate({'val': df_val, 'mygroup': df_group})
    n = 100
    df_val = pd.DataFrame(np.arange(n).reshape(2, -1))
    df_group = pd.DataFrame(np.array([1] * 25 + [2] * 25 + [2] * 20 + [3] * 20 + [9] * 10).reshape(2, -1))
    expr = parser.parse('GroupQuantile(val, mygroup, 5)')
    res = parser.evaluate({'val': df_val, 'mygroup': df_group})
    n1 = 5
    n2 = 4
    n3 = 2
    res_correct = np.array([0.] * n1 + [1.] * n1 + [2.] * n1 + [3.] * n1 + [4.] * n1
                           + [0.] * n1 + [1.] * n1 + [2.] * n1 + [3.] * n1 + [4.] * n1
                           + [0.] * n2 + [1.] * n2 + [2.] * n2 + [3.] * n2 + [4.] * n2
                           + [0.] * n2 + [1.] * n2 + [2.] * n2 + [3.] * n2 + [4.] * n2
                           + [0.] * n3 + [1.] * n3 + [2.] * n3 + [3.] * n3 + [4.] * n3).reshape(2, -1) + 1.0
    assert np.abs(res.values - res_correct).flatten().sum() < 1e-6


def test_quantile():
    val = pd.DataFrame(np.random.rand(500, 3000))
    expr = parser.parse('Quantile(val, 12)')
    res = parser.evaluate({'val': val})
    assert np.nanmean(val[res == 1].values.flatten()) < 0.11

    val = pd.DataFrame(np.random.rand(2000, 100))
    expr = parser.parse('Ts_Quantile(val, 500, 12)')
    res = parser.evaluate({'val': val})
    assert np.nanmean(val[res == 1].values.flatten()) < 0.11
    
    
def test_logical_and_or():
    parser.parse('open + 3 && 1')
    res = parser.evaluate({'open': dfx})
    assert np.all(res.values.flatten())

    parser.parse('open + 3 && 0.0')
    res = parser.evaluate({'open': dfx})
    assert not np.all(res.values.flatten())

    
def test_skew():
    # parser.set_capital('lower')
    expression = parser.parse('Ts_Skewness(open,4)')
    res = parser.evaluate({'close': dfy, 'open': dfx})
    # parser.set_capital('upper')


def test_variables():
    expression = parser.parse('Ts_Skewness(open,4)+close / what')
    res = set(expression.variables()) == {'open', 'close', 'what'}
    
    
def test_product():
    # parser.set_capital('lower')
    expression = parser.parse('Product(open,2)')
    res = parser.evaluate({'close': dfy, 'open': dfx})
    # parser.set_capital('upper')


def test_rank():
    expression = parser.parse('Rank(close)')
    res = parser.evaluate({'close': dfy, 'open': dfx})


def test_tail():
    expression = parser.parse('Tail(close/open,0.99,1.01,1.0)')
    res = parser.evaluate({'close': dfy, 'open': dfx})


def test_step():
    expression = parser.parse('Step(close,10)')
    res = parser.evaluate({'close': dfy, 'open': dfx})


def test_decay_linear():
    expression = parser.parse('Decay_linear(open,2)')
    res = parser.evaluate({'close': dfy, 'open': dfx})


def test_decay_exp():
    expression = parser.parse('Decay_exp(open, 0.5, 2)')
    res = parser.evaluate({'close': dfy, 'open': dfx})


def test_signed_power():
    expression = parser.parse('SignedPower(close-open, 2)')
    res = parser.evaluate({'close': dfx, 'open': dfy})


def test_ewma():
    expr = parser.parse('Ewma(close, 3)')
    res = parser.evaluate({'close': dfx})
    assert abs(res.loc[20170801, '000001.SH'] - 3292.6) < 1e-1


def test_if():
    expr = parser.parse('If(close > 20, 3, -3)')
    res = parser.evaluate({'close': dfx})
    assert res.iloc[0, 0] == 3.
    assert res.iloc[0, 2] == -3.


'''
def test_group_apply():
    import numpy as np
    np.random.seed(369)
    
    n = 20
    
    dic = {c: np.random.rand(n) for c in 'abcdefghijklmnopqrstuvwxyz'[:n]}
    df_value = pd.DataFrame(index=range(n), data=dic)
    
    r = np.random.randint(0, 5, n * df_value.shape[0]).reshape(df_value.shape[0], n)
    cols = df_value.columns.values.copy()
    np.random.shuffle(cols)
    
    df_group = pd.DataFrame(index=df_value.index, columns=cols, data=r)

    parser = Parser()
    expr = parser.parse('GroupApply(Standardize, GroupApply(Cutoff, close, 2.8))')
    res = parser.evaluate({'close': df_value}, df_group=df_group)
    
    assert abs(res.iloc[3, 6] - (-1.53432)) < 1e-5
    assert abs(res.iloc[19, 18] - (-1.17779)) < 1e-5


'''
def test_calc_return():
    expr = parser.parse('Return(close, 2, 0)')
    res = parser.evaluate({'close': dfx})
    assert abs(res.loc[20170808, '000001.SH'] - 0.006067) < 1e-6


@pytest.fixture(autouse=True)
def my_globals(request):
    ds = RemoteDataService()
    ds.init_from_config(data_config)
    
    df, msg = ds.daily("000001.SH, 600030.SH, 000300.SH", start_date=20170801, end_date=20170820,
                       fields="open,high,low,close,vwap,preclose")
    
    multi_index_names = ['trade_date', 'symbol']
    df_multi = df.set_index(multi_index_names, drop=False)
    df_multi.sort_index(axis=0, level=multi_index_names, inplace=True)
    
    dfx = df_multi.loc[pd.IndexSlice[:, :], pd.IndexSlice['close']].unstack()
    dfy = df_multi.loc[pd.IndexSlice[:, :], pd.IndexSlice['open']].unstack()
    
    parser = Parser()
    request.function.func_globals.update({'parser': parser, 'dfx': dfx, 'dfy': dfy})


if __name__ == "__main__":
    ds = RemoteDataService()
    ds.init_from_config(data_config)
    
    df, msg = ds.daily("000001.SH, 600030.SH, 000300.SH", start_date=20170801, end_date=20170820,
                       fields="open,high,low,close,vwap,preclose")
    ds.data_api.close()
    
    multi_index_names = ['trade_date', 'symbol']
    df_multi = df.set_index(multi_index_names, drop=False)
    df_multi.sort_index(axis=0, level=multi_index_names, inplace=True)
    
    dfx = df_multi.loc[pd.IndexSlice[:, :], pd.IndexSlice['close']].unstack()
    dfy = df_multi.loc[pd.IndexSlice[:, :], pd.IndexSlice['open']].unstack()
    
    parser = Parser()
    
    g = globals()
    g = {k: v for k, v in g.viewitems() if k.startswith('test_') and callable(v)}
    
    for test_name, test_func in g.viewitems():
        print "\nTesting {:s}...".format(test_name)
        # try:
        test_func()
        # print "Successfully tested {:s}.".format(test_name)
        # except Exception, e:
    print "Test Complete."



# encoding: utf-8
"""
How to add custom (alternative) data:
    1. Build a DataFrame of your data, whose index is dv.dates, column is dv.symbol.
    2. Use dv.append_df to add your DataFrame to the DataView object.
    
If you will use this data frequently, you can add define a new method of DataServer, then get data from your DataServer.
If you want to declare your field in props, instead of append it manually, you will have to modify prepare_data function.

"""
import os

import numpy as np
import pandas as pd

import jaqs.util as jutil
from jaqs.data.align import align
from jaqs.data.py_expression_eval import Parser


class DataView(object):
    """
    Prepare data before research / trade. Support file I/O.
    Support: add field, add formula, save / load.
    
    Attributes
    ----------
    data_api : RemoteDataService
    symbol : list
    start_date : int
    end_date : int
    fields : list
    freq : int
    market_daily_fields, reference_daily_fields : list
    data_d : pd.DataFrame
        All daily frequency data will be merged and stored here.
        index is date, columns is symbol-field MultiIndex
    data_q : pd.DataFrame
        All quarterly frequency data will be merged and stored here.
        index is date, columns is symbol-field MultiIndex
    
    """
    def __init__(self):
        self.data_api = None
        
        self.universe = ""
        self.symbol = []
        self.start_date = 0
        self.extended_start_date_d = 0
        self.extended_start_date_q = 0
        self.end_date = 0
        self.fields = []
        self.freq = 1
        self.all_price = True

        self.meta_data_list = ['start_date', 'end_date',
                               'extended_start_date_d', 'extended_start_date_q',
                               'freq', 'fields', 'symbol', 'universe',
                               'custom_daily_fields', 'custom_quarterly_fields']
        self.adjust_mode = 'post'
        
        self.data_d = None
        self.data_q = None
        self._data_benchmark = None
        self._data_inst = None
        # self._data_group = None
        
        common_list = {'symbol', 'start_date', 'end_date'}
        market_bar_list = {'open', 'high', 'low', 'close', 'volume', 'turnover', 'vwap', 'oi'}
        market_tick_list = {'volume', 'oi',
                            'askprice1', 'askprice2', 'askprice3', 'askprice4', 'askprice5',
                            'bidprice1', 'bidprice1', 'bidprice1', 'bidprice1', 'bidprice1',
                            'askvolume1', 'askvolume2', 'askvolume3', 'askvolume4', 'askvolume5',
                            'bidvolume1', 'bidvolume2', 'bidvolume3', 'bidvolume4', 'bidvolume5'}
        # fields map
        # TODO: 'freq' is not in market_daily_fields yet.
        self.market_daily_fields = \
            {'open', 'high', 'low', 'close', 'volume', 'turnover', 'vwap', 'oi', 'trade_status',
             'open_adj', 'high_adj', 'low_adj', 'close_adj', 'vwap_adj', 'index_member', 'index_weight'}
        self.group_fields = {'sw1', 'sw2', 'sw3', 'sw4', 'zz1', 'zz2'}
        self.reference_daily_fields = \
            {"total_mv", "float_mv", "pe", "pb", "pe_ttm", "pcf_ocf", "pcf_ocfttm", "pcf_ncf",
             "pcf_ncfttm", "ps", "ps_ttm", "turnover_ratio", "free_turnover_ratio", "total_share",
             "float_share", "price_div_dps", "free_share", "np_parent_comp_ttm",
             "np_parent_comp_lyr", "net_assets", "ncf_oper_ttm", "ncf_oper_lyr", "oper_rev_ttm",
             "oper_rev_lyr", "limit_status"}
        self.fin_stat_income = \
            {"symbol", "ann_date", "start_date", "end_date",
             "comp_type_code", "comp_type_code", "act_ann_date", "start_actdate",
             "end_actdate", "report_date", "start_reportdate", "start_reportdate",
             "report_type", "report_type", "currency", "total_oper_rev", "oper_rev",
             "int_income", "net_int_income", "insur_prem_unearned",
             "handling_chrg_income", "net_handling_chrg_income", "net_inc_other_ops",
             "plus_net_inc_other_bus", "prem_income", "less_ceded_out_prem",
             "chg_unearned_prem_res", "incl_reinsurance_prem_inc",
             "net_inc_sec_trading_brok_bus", "net_inc_sec_uw_bus",
             "net_inc_ec_asset_mgmt_bus", "other_bus_income", "plus_net_gain_chg_fv",
             "plus_net_invest_inc", "incl_inc_invest_assoc_jv_entp",
             "plus_net_gain_fx_trans", "tot_oper_cost", "less_oper_cost", "less_int_exp",
             "less_handling_chrg_comm_exp", "less_taxes_surcharges_ops",
             "less_selling_dist_exp", "less_gerl_admin_exp", "less_fin_exp",
             "less_impair_loss_assets", "prepay_surr", "tot_claim_exp",
             "chg_insur_cont_rsrv", "dvd_exp_insured", "reinsurance_exp", "oper_exp",
             "less_claim_recb_reinsurer", "less_ins_rsrv_recb_reinsurer",
             "less_exp_recb_reinsurer", "other_bus_cost", "oper_profit",
             "plus_non_oper_rev", "less_non_oper_exp", "il_net_loss_disp_noncur_asset",
             "tot_profit", "inc_tax", "unconfirmed_invest_loss",
             "net_profit_incl_min_int_inc", "net_profit_excl_min_int_inc",
             "minority_int_inc", "other_compreh_inc", "tot_compreh_inc",
             "tot_compreh_inc_parent_comp", "tot_compreh_inc_min_shrhldr", "ebit",
             "ebitda", "net_profit_after_ded_nr_lp", "net_profit_under_intl_acc_sta",
             "eps_basic", "eps_diluted", "insurance_expense",
             "spe_bal_oper_profit", "tot_bal_oper_profit", "spe_bal_tot_profit",
             "tot_bal_tot_profit", "spe_bal_net_profit", "tot_bal_net_profit",
             "undistributed_profit", "adjlossgain_prevyear",
             "transfer_from_surplusreserve", "transfer_from_housingimprest",
             "transfer_from_others", "distributable_profit", "withdr_legalsurplus",
             "withdr_legalpubwelfunds", "workers_welfare", "withdr_buzexpwelfare",
             "withdr_reservefund", "distributable_profit_shrhder", "prfshare_dvd_payable",
             "withdr_othersurpreserve", "comshare_dvd_payable",
             "capitalized_comstock_div"}
        self.fin_stat_balance_sheet = \
            {"monetary_cap", "tradable_assets", "notes_rcv", "acct_rcv", "other_rcv", "pre_pay",
             "dvd_rcv", "int_rcv", "inventories", "consumptive_assets", "deferred_exp",
             "noncur_assets_due_1y", "settle_rsrv", "loans_to_banks", "prem_rcv", "rcv_from_reinsurer",
             "rcv_from_ceded_insur_cont_rsrv", "red_monetary_cap_for_sale", "other_cur_assets",
             "tot_cur_assets", "fin_assets_avail_for_sale", "held_to_mty_invest", "long_term_eqy_invest",
             "invest_real_estate", "time_deposits", "other_assets", "long_term_rec", "fix_assets",
             "const_in_prog", "proj_matl", "fix_assets_disp", "productive_bio_assets",
             "oil_and_natural_gas_assets", "intang_assets", "r_and_d_costs", "goodwill",
             "long_term_deferred_exp", "deferred_tax_assets", "loans_and_adv_granted",
             "oth_non_cur_assets", "tot_non_cur_assets", "cash_deposits_central_bank",
             "asset_dep_oth_banks_fin_inst", "precious_metals", "derivative_fin_assets",
             "agency_bus_assets", "subr_rec", "rcv_ceded_unearned_prem_rsrv", "rcv_ceded_claim_rsrv",
             "rcv_ceded_life_insur_rsrv", "rcv_ceded_lt_health_insur_rsrv", "mrgn_paid",
             "insured_pledge_loan", "cap_mrgn_paid", "independent_acct_assets", "clients_cap_deposit",
             "clients_rsrv_settle", "incl_seat_fees_exchange", "rcv_invest", "tot_assets", "st_borrow",
             "borrow_central_bank", "deposit_received_ib_deposits", "loans_oth_banks", "tradable_fin_liab",
             "notes_payable", "acct_payable", "adv_from_cust", "fund_sales_fin_assets_rp",
             "handling_charges_comm_payable", "empl_ben_payable", "taxes_surcharges_payable", "int_payable",
             "dvd_payable", "other_payable", "acc_exp", "deferred_inc", "st_bonds_payable", "payable_to_reinsurer",
             "rsrv_insur_cont", "acting_trading_sec", "acting_uw_sec", "non_cur_liab_due_within_1y", "other_cur_liab",
             "tot_cur_liab", "lt_borrow", "bonds_payable", "lt_payable", "specific_item_payable", "provisions",
             "deferred_tax_liab", "deferred_inc_non_cur_liab", "other_non_cur_liab", "tot_non_cur_liab",
             "liab_dep_other_banks_inst", "derivative_fin_liab", "cust_bank_dep", "agency_bus_liab", "other_liab",
             "prem_received_adv", "deposit_received", "insured_deposit_invest", "unearned_prem_rsrv", "out_loss_rsrv",
             "life_insur_rsrv", "lt_health_insur_v", "independent_acct_liab", "incl_pledge_loan", "claims_payable",
             "dvd_payable_insured", "total_liab", "capital_stk", "capital_reser", "special_rsrv", "surplus_rsrv",
             "undistributed_profit", "less_tsy_stk", "prov_nom_risks", "cnvd_diff_foreign_curr_stat",
             "unconfirmed_invest_loss", "minority_int", "tot_shrhldr_eqy_excl_min_int", "tot_shrhldr_eqy_incl_min_int",
             "tot_liab_shrhldr_eqy", "spe_cur_assets_diff", "tot_cur_assets_diff", "spe_non_cur_assets_diff",
             "tot_non_cur_assets_diff", "spe_bal_assets_diff", "tot_bal_assets_diff", "spe_cur_liab_diff",
             "tot_cur_liab_diff", "spe_non_cur_liab_diff", "tot_non_cur_liab_diff", "spe_bal_liab_diff",
             "tot_bal_liab_diff", "spe_bal_shrhldr_eqy_diff", "tot_bal_shrhldr_eqy_diff", "spe_bal_liab_eqy_diff",
             "tot_bal_liab_eqy_diff", "lt_payroll_payable", "other_comp_income", "other_equity_tools",
             "other_equity_tools_p_shr", "lending_funds", "accounts_receivable", "st_financing_payable", "payables"}
        self.fin_stat_cash_flow = \
            {"cash_recp_sg_and_rs", "recp_tax_rends", "net_incr_dep_cob", "net_incr_loans_central_bank",
             "net_incr_fund_borr_ofi", "cash_recp_prem_orig_inco", "net_incr_insured_dep",
             "net_cash_received_reinsu_bus", "net_incr_disp_tfa", "net_incr_int_handling_chrg", "net_incr_disp_faas",
             "net_incr_loans_other_bank", "net_incr_repurch_bus_fund", "other_cash_recp_ral_oper_act",
             "stot_cash_inflows_oper_act", "cash_pay_goods_purch_serv_rec", "cash_pay_beh_empl", "pay_all_typ_tax",
             "net_incr_clients_loan_adv", "net_incr_dep_cbob", "cash_pay_claims_orig_inco", "handling_chrg_paid",
             "comm_insur_plcy_paid", "other_cash_pay_ral_oper_act", "stot_cash_outflows_oper_act",
             "net_cash_flows_oper_act", "cash_recp_disp_withdrwl_invest", "cash_recp_return_invest",
             "net_cash_recp_disp_fiolta", "net_cash_recp_disp_sobu", "other_cash_recp_ral_inv_act",
             "stot_cash_inflows_inv_act", "cash_pay_acq_const_fiolta", "cash_paid_invest", "net_cash_pay_aquis_sobu",
             "other_cash_pay_ral_inv_act", "net_incr_pledge_loan", "stot_cash_outflows_inv_act",
             "net_cash_flows_inv_act", "cash_recp_cap_contrib", "incl_cash_rec_saims", "cash_recp_borrow",
             "proc_issue_bonds", "other_cash_recp_ral_fnc_act", "stot_cash_inflows_fnc_act", "cash_prepay_amt_borr",
             "cash_pay_dist_dpcp_int_exp", "incl_dvd_profit_paid_sc_ms", "other_cash_pay_ral_fnc_act",
             "stot_cash_outflows_fnc_act", "net_cash_flows_fnc_act", "eff_fx_flu_cash", "net_incr_cash_cash_equ",
             "cash_cash_equ_beg_period", "cash_cash_equ_end_period", "net_profit", "unconfirmed_invest_loss",
             "plus_prov_depr_assets", "depr_fa_coga_dpba", "amort_intang_assets", "amort_lt_deferred_exp",
             "decr_deferred_exp", "incr_acc_exp", "loss_disp_fiolta", "loss_scr_fa", "loss_fv_chg", "fin_exp",
             "invest_loss", "decr_deferred_inc_tax_assets", "incr_deferred_inc_tax_liab", "decr_inventories",
             "decr_oper_payable", "incr_oper_payable", "others", "im_net_cash_flows_oper_act", "conv_debt_into_cap",
             "conv_corp_bonds_due_within_1y", "fa_fnc_leases", "end_bal_cash", "less_beg_bal_cash",
             "plus_end_bal_cash_equ", "less_beg_bal_cash_equ", "im_net_incr_cash_cash_equ", "free_cash_flow",
             "spe_bal_cash_inflows_oper", "tot_bal_cash_inflows_oper", "spe_bal_cash_outflows_oper",
             "tot_bal_cash_outflows_oper", "tot_bal_netcash_outflows_oper", "spe_bal_cash_inflows_inv",
             "tot_bal_cash_inflows_inv", "spe_bal_cash_outflows_inv", "tot_bal_cash_outflows_inv",
             "tot_bal_netcash_outflows_inv", "spe_bal_cash_inflows_fnc", "tot_bal_cash_inflows_fnc",
             "spe_bal_cash_outflows_fnc", "tot_bal_cash_outflows_fnc", "tot_bal_netcash_outflows_fnc",
             "spe_bal_netcash_inc", "tot_bal_netcash_inc", "spe_bal_netcash_equ_undir", "tot_bal_netcash_equ_undir",
             "spe_bal_netcash_inc_undir", "tot_bal_netcash_inc_undir"}
        self.fin_indicator = \
            {"extraordinary","deductedprofit","grossmargin","operateincome","investincome","stmnote_finexp",
             "stm_is","ebit","ebitda""fcff","fcfe","exinterestdebt_current","exinterestdebt_noncurrent","interestdebt",
             "netdebt","tangibleasset","workingcapital","networkingcapital","investcapital","retainedearnings","eps_basic_daily", # TODO eps_basic
             "eps_diluted","eps_diluted2","bps","ocfps","grps","orps","surpluscapitalps","surplusreserveps","undistributedps",
             "retainedps","cfps","ebitps","fcffps","fcfeps","netprofitmargin","grossprofitmargin","cogstosales",
             "expensetosales","profittogr","saleexpensetogr","adminexpensetogr","finaexpensetogr","impairtogr_ttm",
             "gctogr","optogr","ebittogr","roe","roe_deducted","roa2","roa","roic","roe_yearly","roa2_yearly","roe_avg",
             "operateincometoebt","investincometoebt","nonoperateprofittoebt","taxtoebt","deductedprofittoprofit","salescashintoor",
             "ocftoor","ocftooperateincome","capitalizedtoda","debttoassets","assetstoequity","dupont_assetstoequity",
             "catoassets","ncatoassets","tangibleassetstoassets","intdebttototalcap","equitytototalcapital","currentdebttodebt",
             "longdebtodebt","current","quick","cashratio","ocftoshortdebt","debttoequity","equitytodebt",
             "equitytointerestdebt","tangibleassettodebt","tangassettointdebt","tangibleassettonetdebt","ocftodebt",
             "ocftointerestdebt","ocftonetdebt","ebittointerest","longdebttoworkingcapital","ebitdatodebt","turndays",
             "invturndays","arturndays","invturn","arturn","caturn","faturn","assetsturn","roa_yearly","dupont_roa",
             "s_stm_bs","prefinexpense_opprofit","nonopprofit","optoebt","noptoebt","ocftoprofit","cashtoliqdebt",
             "cashtoliqdebtwithinterest","optoliqdebt","optodebt","roic_yearly","tot_faturn","profittoop","qfa_operateincome",
             "qfa_investincome","qfa_deductedprofit","qfa_eps","qfa_netprofitmargin","qfa_grossprofitmargin","qfa_expensetosales",
             "qfa_profittogr","qfa_saleexpensetogr","qfa_adminexpensetogr","qfa_finaexpensetogr","qfa_impairtogr_ttm",
             "qfa_gctogr","qfa_optogr","qfa_roe","qfa_roe_deducted","qfa_roa","qfa_operateincometoebt","qfa_investincometoebt",
             "qfa_deductedprofittoprofit","qfa_salescashintoor","qfa_ocftosales","qfa_ocftoor","yoyeps_basic","yoyeps_diluted",
             "yoyocfps","yoyop","yoyebt","yoynetprofit","yoynetprofit_deducted","yoyocf","yoyroe","yoybps","yoyassets",
             "yoyequity","yoy_tr","yoy_or","qfa_yoygr","qfa_cgrgr","qfa_yoysales","qfa_cgrsales","qfa_yoyop","qfa_cgrop",
             "qfa_yoyprofit","qfa_cgrprofit","qfa_yoynetprofit","qfa_cgrnetprofit","yoy_equity","rd_expense","waa_roe"}
        self .custom_daily_fields = []
        self .custom_quarterly_fields = []
        
        # co nst
        self .ANN_DATE_FIELD_NAME = 'ann_date'
        self .REPORT_DATE_FIELD_NAME = 'report_date'
        self.TRADE_STATUS_FIELD_NAME = 'trade_status'
        self.TRADE_DATE_FIELD_NAME = 'trade_date'
    
    # --------------------------------------------------------------------------------------------------------
    # Properties
    @property
    def data_benchmark(self):
        return self._data_benchmark
    
    @property
    def data_inst(self):
        """
        
        Returns
        -------
        pd.DataFrame

        """
        return self._data_inst
    
    @data_benchmark.setter
    def data_benchmark(self, df_new):
        if self.data_d is not None and df_new.shape[0] != self.data_d.shape[0]:
            raise ValueError("You must provide a DataFrame with the same shape of data_benchmark.")
        self._data_benchmark = df_new

    @property
    def dates(self):
        """
        Get daily date array of the underlying data.
        
        Returns
        -------
        res : np.array
            dtype: int

        """
        if self.data_d is not None:
            res = self.data_d.index.values
        elif self.data_api is not None:
            res = self.data_api.get_trade_date_range(self.extended_start_date_d, self.end_date)
        else:
            raise ValueError("Cannot get dates array when neither of data and data_api exists.")
    
        return res

    # --------------------------------------------------------------------------------------------------------
    # Fields
    def _is_quarter_field(self, field_name):
        """
        Check whether a field name is quarterly frequency.
        field_name must be pre-defined or already added.
        
        Parameters
        ----------
        field_name : str

        Returns
        -------
        bool

        """
        res = (field_name in self.fin_stat_balance_sheet
               or field_name in self.fin_stat_cash_flow
               or field_name in self.fin_stat_income
               or field_name in self.fin_indicator
               or field_name in self.custom_quarterly_fields)
        return res

    def _is_daily_field(self, field_name):
        """
        Check whether a field name is daily frequency.
        field_name must be pre-defined or already added.
        
        Parameters
        ----------
        field_name : str

        Returns
        -------
        bool

        """
        flag = (field_name in self.market_daily_fields
                or field_name in self.reference_daily_fields
                or field_name in self.custom_daily_fields
                or field_name in self.group_fields)
        return flag

    def _is_predefined_field(self, field_name):
        """
        Check whether a field name can be recognized.
        field_name must be pre-defined or already added.
        
        Parameters
        ----------
        field_name : str

        Returns
        -------
        bool

        """
        return self._is_quarter_field(field_name) or self._is_daily_field(field_name)

    def _get_fields(self, field_type, fields, complement=False, append=False):
        """
        Get list of fields that are in ref_quarterly_fields.
        
        Parameters
        ----------
        field_type : {'market_daily', 'ref_daily', 'income', 'balance_sheet', 'cash_flow', 'daily', 'quarterly'
        fields : list of str
        complement : bool, optional
            If True, get fields that are NOT in ref_quarterly_fields.

        Returns
        -------
        list

        """
        pool_map = {'market_daily': self.market_daily_fields,
                    'ref_daily': self.reference_daily_fields,
                    'income': self.fin_stat_income,
                    'balance_sheet': self.fin_stat_balance_sheet,
                    'cash_flow': self.fin_stat_cash_flow,
                    'fin_indicator': self.fin_indicator,
                    'group': self.group_fields}
        pool_map['daily'] = set.union(pool_map['market_daily'],
                                      pool_map['ref_daily'],
                                      pool_map['group'],
                                      self.custom_daily_fields)
        pool_map['quarterly'] = set.union(pool_map['income'],
                                          pool_map['balance_sheet'],
                                          pool_map['cash_flow'],
                                          pool_map['fin_indicator'],
                                          self.custom_quarterly_fields)
        
        pool = pool_map.get(field_type, None)
        if pool is None:
            raise NotImplementedError("field_type = {:s}".format(field_type))
        
        s = set.intersection(set(pool), set(fields))
        if not s:
            return []
        
        if complement:
            s = set(fields) - s
            
        if field_type == 'market_daily' and self.all_price:
            # turnover will not be adjusted
            s.update({'open', 'high', 'close', 'low', 'vwap'})
            
        if append:
            s.add('symbol')
            if field_type == 'market_daily' or field_type == 'ref_daily':
                s.add('trade_date')
                if field_type == 'market_daily':
                    s.add(self.TRADE_STATUS_FIELD_NAME)
            elif (field_type == 'income'
                  or field_type == 'balance_sheet'
                  or field_type == 'cash_flow'
                  or field_type == 'fin_indicator'):
                s.add(self.ANN_DATE_FIELD_NAME)
                s.add(self.REPORT_DATE_FIELD_NAME)
        
        l = list(s)
        return l

    # --------------------------------------------------------------------------------------------------------
    # Prepare data
    def init_from_config(self, props, data_api):
        """
        Initialize various attributes like start/end date, universe/symbol, fields, etc.
        If your want to parse symbol, but use a custom benchmark index, please directly assign self.data_benchmark.
        
        Parameters
        ----------
        props : dict
            start_date, end_date, freq, symbol, fields, etc.
        data_api : BaseDataServer
        
        """
        # data_api.init_from_config(props)
        self.data_api = data_api
    
        sep = ','
    
        # initialize parameters
        self.start_date = props['start_date']
        self.extended_start_date_d = jutil.shift(self.start_date, n_weeks=-8)  # query more data
        self.extended_start_date_q = jutil.shift(self.start_date, n_weeks=-80)
        self.end_date = props['end_date']
        self.all_price = props.get('all_price', True)
        self.freq = props.get('freq', 1)
    
        # get and filter fields
        fields = props.get('fields', [])
        if fields:
            fields = props['fields'].split(sep)
            self.fields = [field for field in fields if self._is_predefined_field(field)]
            if len(self.fields) < len(fields):
                print "Field name [{}] not valid, ignore.".format(set.difference(set(fields), set(self.fields)))
    
        # append additional fields
        if self.all_price:
            self.fields.extend(['open_adj', 'high_adj', 'low_adj', 'close_adj',
                                'open', 'high', 'low', 'close',
                                'vwap', 'vwap_adj'])
    
        # initialize universe/symbol
        universe = props.get('universe', "")
        symbol = props.get('symbol', "")
        if symbol and universe:
            raise ValueError("Please use either [symbol] or [universe].")
        if not (symbol or universe):
            raise ValueError("One of [symbol] or [universe] must be provided.")
        if universe:
            self.universe = universe
            self.symbol = sorted(data_api.get_index_comp(self.universe, self.extended_start_date_d, self.end_date))
        else:
            self.symbol = sorted(symbol.split(sep))
    
        print "Initialize config success."

    def prepare_data(self):
        """Prepare data for the FIRST time."""
        # prepare benchmark and group
        print "Query data..."
        data_d, data_q = self._prepare_daily_quarterly(self.fields)
        self.data_d, self.data_q = data_d, data_q
        self._align_and_merge_q_into_d()
    
        print "Query instrument info..."
        self._prepare_inst_info()
    
        print "Query adj_factor..."
        self._prepare_adj_factor()
    
        if self.universe:
            print "Query benchmark..."
            self._data_benchmark = self._prepare_benchmark()
            print "Query benchmar member info..."
            self._prepare_comp_info()
    
        group_fields = self._get_fields('group', self.fields)
        if group_fields:
            print "Query groups (industry)..."
            self._prepare_group(group_fields)
    
        print "Data has been successfully prepared."

    @staticmethod
    def _process_index_co(df, index_name):
        df = df.astype(dtype={index_name: int})
        df = df.drop_duplicates(subset=['symbol', index_name])
        return df

    def _prepare_daily_quarterly(self, fields):
        """
        Query and process data from data_api.
        
        Parameters
        ----------
        fields : list

        Returns
        -------
        merge_d : pd.DataFrame or None
        merge_q : pd.DataFrame or None

        """
        if not fields:
            return None, None
    
        # query data
        print "Query data - query..."
        daily_list, quarterly_list = self._query_data(self.symbol, fields)
    
        def pivot_and_sort(df, index_name):
            df = self._process_index_co(df, index_name)
            df = df.pivot(index=index_name, columns='symbol')
            df.columns = df.columns.swaplevel()
            col_names = ['symbol', 'field']
            df.columns.names = col_names
            df = df.sort_index(axis=1, level=col_names)
            df.index.name = index_name
            return df
    
        multi_daily = None
        multi_quarterly = None
        if daily_list:
            daily_list_pivot = [pivot_and_sort(df, self.TRADE_DATE_FIELD_NAME) for df in daily_list]
            multi_daily = self._merge_data(daily_list_pivot, self.TRADE_DATE_FIELD_NAME)
            # use self.dates as index because original data have weekends
            multi_daily = self._fill_missing_idx_col(multi_daily, index=self.dates, symbols=self.symbol)
            print "Query data - daily fields prepared."
        if quarterly_list:
            quarterly_list_pivot = [pivot_and_sort(df, self.REPORT_DATE_FIELD_NAME) for df in quarterly_list]
            multi_quarterly = self._merge_data(quarterly_list_pivot, self.REPORT_DATE_FIELD_NAME)
            multi_quarterly = self._fill_missing_idx_col(multi_quarterly, index=None, symbols=self.symbol)
            print "Query data - quarterly fields prepared."
    
        return multi_daily, multi_quarterly

    def _query_data(self, symbol, fields):
        """
        Query data using different APIs, then store them in dict.
        period, start_date and end_date are fixed.
        Keys of dict are securitites.
        
        Parameters
        ----------
        symbol : list of str
        fields : list of str

        Returns
        -------
        daily_list : list
        quarterly_list : list

        """
        sep = ','
        symbol_str = sep.join(symbol)
    
        if self.freq == 1:
            daily_list = []
            quarterly_list = []
        
            # TODO : use fields = {field: kwargs} to enable params
            fields_market_daily = self._get_fields('market_daily', fields, append=True)
            if fields_market_daily:
                print "NOTE: price adjust method is [{:s} adjust]".format(self.adjust_mode)
                # no adjust prices and other market daily fields
                df_daily, msg1 = self.data_api.daily(symbol_str, start_date=self.extended_start_date_d, end_date=self.end_date,
                                                     adjust_mode=None, fields=sep.join(fields_market_daily))
                if msg1 != '0,':
                    print msg1
            
                if self.all_price:
                    adj_cols = ['open', 'high', 'low', 'close', 'vwap']
                    # adjusted prices
                    df_daily_adjust, msg11 = self.data_api.daily(symbol_str, start_date=self.extended_start_date_d, end_date=self.end_date,
                                                                 adjust_mode=self.adjust_mode, fields=','.join(adj_cols))
                    if msg11 != '0,':
                        print msg11
                
                    df_daily = pd.merge(df_daily, df_daily_adjust, how='outer',
                                        on=['symbol', 'trade_date'], suffixes=('', '_adj'))
                daily_list.append(df_daily.loc[:, fields_market_daily])
        
            fields_ref_daily = self._get_fields('ref_daily', fields, append=True)
            if fields_ref_daily:
                df_ref_daily, msg2 = self.data_api.query_lb_dailyindicator(symbol_str, self.extended_start_date_d, self.end_date,
                                                                           sep.join(fields_ref_daily))
                if msg2 != '0,':
                    print msg2
                daily_list.append(df_ref_daily.loc[:, fields_ref_daily])
        
            fields_income = self._get_fields('income', fields, append=True)
            if fields_income:
                df_income, msg3 = self.data_api.query_lb_fin_stat('income', symbol_str, self.extended_start_date_q, self.end_date,
                                                                  sep.join(fields_income), drop_dup_cols=['symbol', self.REPORT_DATE_FIELD_NAME])
                if msg3 != '0,':
                    print msg3
                quarterly_list.append(df_income.loc[:, fields_income])
        
            fields_balance = self._get_fields('balance_sheet', fields, append=True)
            if fields_balance:
                df_balance, msg3 = self.data_api.query_lb_fin_stat('balance_sheet', symbol_str, self.extended_start_date_q, self.end_date,
                                                                   sep.join(fields_balance), drop_dup_cols=['symbol', self.REPORT_DATE_FIELD_NAME])
                if msg3 != '0,':
                    print msg3
                quarterly_list.append(df_balance.loc[:, fields_balance])
        
            fields_cf = self._get_fields('cash_flow', fields, append=True)
            if fields_cf:
                df_cf, msg3 = self.data_api.query_lb_fin_stat('cash_flow', symbol_str, self.extended_start_date_q, self.end_date,
                                                              sep.join(fields_cf), drop_dup_cols=['symbol', self.REPORT_DATE_FIELD_NAME])
                if msg3 != '0,':
                    print msg3
                quarterly_list.append(df_cf.loc[:, fields_cf])
        
            fields_fin_ind = self._get_fields('fin_indicator', fields, append=True)
            if fields_fin_ind:
                df_fin_ind, msg4 = self.data_api.query_lb_fin_stat('fin_indicator', symbol_str,
                                                                   self.extended_start_date_q, self.end_date,
                                                                   sep.join(fields_cf), drop_dup_cols=['symbol', self.REPORT_DATE_FIELD_NAME])
                if msg4 != '0,':
                    print msg4
                quarterly_list.append(df_fin_ind.loc[:, fields_fin_ind])
    
        else:
            raise NotImplementedError("freq = {}".format(self.freq))
    
        return daily_list, quarterly_list

    '''
    @staticmethod
    def _process_index(df, index_name='trade_date'):
        """
        Drop duplicates, set and sort index.
        
        Parameters
        ----------
        df : pd.DataFrame
            index of df must be unique.
        index_name : str, optional
            label of column which will be used as index.

        Returns
        -------
        df : pd.DataFrame
            processed
        
        Notes
        -----
        We do not use inplace operations, which will be a lot slower

        """
        # df.drop_duplicates(subset=index_name, inplace=True)  # TODO not a good solution
        dtype_idx = df.dtypes[index_name].type
        if not issubclass(dtype_idx, (int, np.integer)):
            df = df.astype(dtype={index_name: int})  # fast data type conversion
        
        dup = df.duplicated(subset=index_name)
        if np.sum(dup.values) > 0:
            # TODO
            print "WARNING: Duplicate {:s} encountered, droped.".format(index_name)
            idx = np.logical_not(dup)
            df = df.loc[idx, :]
        
        df = df.set_index(index_name)
        df = df.sort_index(axis=0)

        if 'symbol' in df.columns:
            df = df.drop(['symbol'], axis=1)
        
        return df

    def _dic_of_df_to_multi_index_df(self, dic, level_names=None):
        """
        Convert dict of DataFrame to MultiIndex DataFrame.
        Columns of result will be MultiIndex constructed using keys of dict and columns of DataFrame.
        Index of result will be the same with DataFrame.
        Different DataFrame will be aligned (outer join) using index.

        Parameters
        ----------
        dic : dict
            {symbol: DataFrame with index be datetime and columns be fields}.
        fields : list or np.ndarray
            Column labels for MultiIndex level 0.
        level_names : list of str
            Name of columns.

        Returns
        -------
        merge : pd.DataFrame
            with MultiIndex columns.

        """
        if level_names is None:
            level_names = ['symbol', 'field']
        merge = pd.concat(dic, axis=1)
        
        values = dic.values()
        idx = np.unique(np.concatenate([df.index.values for df in values]))
        fields = np.unique(np.concatenate([df.columns.values for df in values]))

        cols_multi = pd.MultiIndex.from_product([self.symbol, fields], names=level_names)
        cols_multi = cols_multi.sort_values()
        merge_final = pd.DataFrame(index=idx, columns=cols_multi, data=np.nan)

        merge_final.update(merge)

        if merge_final.isnull().sum().sum() > 0:
            print "WARNING: there are NaN values in your data, NO fill."
            # merge.fillna(method='ffill')

        if merge_final.shape != merge.shape:
            idx_diff = sorted(set(merge_final.index) - set(merge.index))
            col_diff = sorted(set(merge_final.columns.levels[0].values) - set(merge.columns.levels[0].values))
            print ("WARNING: some data is unavailable: "
                   + "\n    At index " + ', '.join(idx_diff)
                   + "\n    At fields " + ', '.join(col_diff))
        return merge_final

    def _preprocess_market_daily(self, dic):
        """
        Process data and construct MultiIndex.
        
        Parameters
        ----------
        dic : dict

        Returns
        -------
        res : pd.DataFrame

        """
        if not dic:
            return None
        
        dic_res = dict()
        n = len(dic)
        for i, (sec, df) in enumerate(dic.items()):
            x = i * 1. / n
            k = int(10 * x)
            if x - 0.1 * k < 1. / n:
                print '.',
            dic_res[sec] = self._process_index(df, self.TRADE_DATE_FIELD_NAME)
        print
        
        res = self._dic_of_df_to_multi_index_df(dic_res, level_names=['symbol', 'field'])
        return res
        
    def _preprocess_ref_daily(self, dic, fields):
        """
        Process data and construct MultiIndex.
        
        Parameters
        ----------
        dic : dict

        Returns
        -------
        res : pd.DataFrame

        """
        if not dic:
            return None
        
        dic_res = dict()
        n = len(dic)
        for i, (sec, df) in enumerate(dic.items()):
            x = i * 1. / n
            k = int(10 * x)
            if x - 0.1 * k < 1. / n:
                print '.',
            
            df_mod = self._process_index(df, self.TRADE_DATE_FIELD_NAME)
            df_mod = df_mod.loc[:, self._get_fields('ref_daily', fields)]
            dic_res[sec] = df_mod
        print
        
        res = self._dic_of_df_to_multi_index_df(dic_res, level_names=['symbol', 'field'])
        return res

    def _preprocess_ref_quarterly(self, type_, dic, fields):
        """
        Process data and construct MultiIndex.
        
        Parameters
        ----------
        dic : dict

        Returns
        -------
        res : pd.DataFrame

        """
        if not dic:
            return None
        
        dic_res = dict()
        n = len(dic)
        for i, (sec, df) in enumerate(dic.items()):
            x = i * 1. / n
            k = int(10 * x)
            if x - 0.1 * k < 1. / n:
                print '.',
            df_mod = df.loc[:, self._get_fields(type_, fields, append=True)]
            df_mod = self._process_index(df_mod, self.REPORT_DATE_FIELD_NAME)
            
            dic_res[sec] = df_mod
        print
    
        res = self._dic_of_df_to_multi_index_df(dic_res, level_names=['symbol', 'field'])
        return res
    
    '''
    @staticmethod
    def _merge_data(dfs, index_name='trade_date'):
        """
        Merge data from different APIs into one DataFrame.
        
        Parameters
        ----------
        dfs : list of pd.DataFrame

        Returns
        -------
        merge : pd.DataFrame or None
            If dfs is empty, return None
        
        Notes
        -----
        Align on date index, concatenate on columns (symbol and fields)
        
        """
        # dfs = [df for df in dfs if df is not None]
    
        merge = pd.concat(dfs, axis=1, join='outer')
    
        # drop duplicated columns. ONE LINE EFFICIENT version
        mask_duplicated = merge.columns.duplicated()
        if np.any(mask_duplicated):
            # print("Duplicated columns found. Dropped.")
            merge = merge.loc[:, ~mask_duplicated]
        
            # if merge.isnull().sum().sum() > 0:
            # print "WARNING: nan in final merged data. NO fill"
            # merge.fillna(method='ffill', inplace=True)
    
        merge = merge.sort_index(axis=1, level=['symbol', 'field'])
        merge.index.name = index_name
    
        return merge

    def _fill_missing_idx_col(self, df, index=None, symbols=None):
        if index is None:
            index = df.index
        if symbols is None:
            symbols = self.symbol
        fields = df.columns.levels[1]
    
        if len(fields) * len(self.symbol) != len(df.columns) or len(index) != len(df.index):
            cols_multi = pd.MultiIndex.from_product([symbols, fields], names=['symbol', 'field'])
            cols_multi = cols_multi.sort_values()
            df_final = pd.DataFrame(index=index, columns=cols_multi, data=np.nan)
            df_final.index.name = df.index.name
        
            df_final.update(df)
        
            # idx_diff = sorted(set(df_final.index) - set(df.index))
            col_diff = sorted(set(df_final.columns.levels[0].values) - set(df.columns.levels[0].values))
            print ("WARNING: some data is unavailable: "
                   # + "\n    At index " + ', '.join(idx_diff)
                   + "\n    At fields " + ', '.join(col_diff))
            return df_final
        else:
            return df

    def _align_and_merge_q_into_d(self):
        data_d, data_q = self.data_d, self.data_q
        if data_d is not None and data_q is not None:
            df_ref_ann = data_q.loc[:, pd.IndexSlice[:, self.ANN_DATE_FIELD_NAME]].copy()
            df_ref_ann.columns = df_ref_ann.columns.droplevel(level='field')
        
            dic_expanded = dict()
            for field_name, df in data_q.groupby(level=1, axis=1):  # by column multiindex fields
                df_expanded = align(df, df_ref_ann, self.dates)
                dic_expanded[field_name] = df_expanded
            df_quarterly_expanded = pd.concat(dic_expanded.values(), axis=1)
            df_quarterly_expanded.index.name = self.TRADE_DATE_FIELD_NAME
        
            data_d_merge = self._merge_data([data_d, df_quarterly_expanded], index_name=self.TRADE_DATE_FIELD_NAME)
            data_d = data_d_merge.loc[data_d.index, :]
        self.data_d = data_d

    def _prepare_adj_factor(self):
        """Query and append daily adjust factor for prices."""
        mask_stocks = self.data_inst['inst_type'] == 1
        if mask_stocks.sum() == 0:
            return
        symbol_stocks = self.data_inst.loc[mask_stocks].index.values
        symbol_str = ','.join(symbol_stocks)
        df_adj = self.data_api.get_adj_factor_daily(symbol_str,
                                                    start_date=self.extended_start_date_d, end_date=self.end_date, div=False)
        self.append_df(df_adj, 'adjust_factor', is_quarterly=False)

    def _prepare_comp_info(self):
        df = self.data_api.get_index_comp_df(self.universe, self.extended_start_date_d, self.end_date)
        self.append_df(df, 'index_member', is_quarterly=False)
    
        df_weights = self.data_api.get_index_weights_daily(self.universe, self.extended_start_date_d, self.end_date)
        self.append_df(df_weights, 'index_weight', is_quarterly=False)

    def _prepare_inst_info(self):
        res = self.data_api.query_inst_info(symbol=','.join(self.symbol),
                                            fields='symbol,inst_type,name,list_date,'
                                                   'delist_date,product,pricetick,multiplier,'
                                                   'buylot,setlot',
                                            inst_type="")
        self._data_inst = res

    def _prepare_group(self, group_fields):
        data_map = {'sw1': ('SW', 1),
                    'sw2': ('SW', 2),
                    'sw3': ('SW', 3),
                    'sw4': ('SW', 4),
                    'zz1': ('ZZ', 1),
                    'zz2': ('ZZ', 2)}
        for field in group_fields:
            type_, level = data_map[field]
            df = self.data_api.get_industry_daily(symbol=','.join(self.symbol),
                                                  start_date=self.extended_start_date_q, end_date=self.end_date,
                                                  type_=type_, level=level)
            self.append_df(df, field, is_quarterly=False)

    def _prepare_benchmark(self):
        df_bench, msg = self.data_api.daily(self.universe,
                                            start_date=self.extended_start_date_d, end_date=self.end_date,
                                            adjust_mode=self.adjust_mode, fields='trade_date,symbol,close,vwap,volume,turnover')
        if msg != '0,':
            raise ValueError("msg = '{:s}'".format(msg))
        
        # TODO: we want more than just close price of benchmark
        df_bench = df_bench.set_index('trade_date').loc[:, ['close']]
        return df_bench

    # --------------------------------------------------------------------------------------------------------
    # Add/Remove Fields&Formulas
    def _add_field(self, field_name, is_quarterly=None):
        self.fields.append(field_name)
        if not self._is_predefined_field(field_name):
            if is_quarterly is None:
                raise ValueError("Field [{:s}] is not a predefined field, but no frequency information is provided.")
            if is_quarterly:
                self.custom_quarterly_fields.append(field_name)
            else:
                self.custom_daily_fields.append(field_name)
    
    def add_field(self, field_name, data_api=None):
        """
        Query and append new field to DataView.
        
        Parameters
        ----------
        data_api : BaseDataServer
        field_name : str
            Must be a known field name (which is given in documents).
        
        Returns
        -------
        bool
            whether add successfully.

        """
        if data_api is None:
            if self.data_api is None:
                print "Add field failed. No data_api available. Please specify one in parameter."
                return False
        else:
            self.data_api = data_api
            
        if field_name in self.fields:
            print "Field name [{:s}] already exists.".format(field_name)
            return False

        if not self._is_predefined_field(field_name):
            print "Field name [{}] not valid, ignore.".format(field_name)
            return False

        merge_d, merge_q = self._prepare_daily_quarterly([field_name])
    
        if self._is_daily_field(field_name):
            if self.data_d is None:
                raise ValueError("Please prepare [{:s}] first.".format(field_name))
            merge, _ = self._prepare_daily_quarterly([field_name])
            is_quarterly = False
        else:
            if self.data_q is None:
                raise ValueError("Please prepare [{:s}] first.".format(field_name))
            _, merge = self._prepare_daily_quarterly([field_name])
            is_quarterly = True
        
        merge = merge.loc[:, pd.IndexSlice[:, field_name]]
        merge.columns = merge.columns.droplevel(level=1)
        self.append_df(merge, field_name, is_quarterly=is_quarterly)  # whether contain only trade days is decided by existing data.
        
        if is_quarterly:
            df_ann = merge_q.loc[:, pd.IndexSlice[:, self.ANN_DATE_FIELD_NAME]]
            df_ann.columns = df_ann.columns.droplevel(level='field')
            df_expanded = align(merge, df_ann, self.dates)
            self.append_df(df_expanded, field_name, is_quarterly=False)
        return True
    
    def add_formula(self, field_name, formula, is_quarterly,
                    formula_func_name_style='camel', data_api=None,
                    within_index=True):
        """
        Add a new field, which is calculated using existing fields.
        
        Parameters
        ----------
        formula : str
            A formula contains operations and function calls.
        field_name : str
            A custom name for the new field.
        is_quarterly : bool
            Whether df is quarterly data (like quarterly financial statement) or daily data.
        formula_func_name_style : {'upper', 'lower'}, optional
        data_api : RemoteDataService, optional
        within_index : bool
            When do cross-section operatioins, whether just do within index components.
        
        """
        if data_api is not None:
            self.data_api = data_api
            
        if field_name in self.fields:
            print "Add formula failed: name [{:s}] exist. Try another name.".format(field_name)
            return
        
        parser = Parser()
        parser.set_capital(formula_func_name_style)
        
        expr = parser.parse(formula)
        
        var_df_dic = dict()
        var_list = expr.variables()
        
        # TODO
        # users do not need to prepare data before add_formula
        if not self.fields:
            self.fields.extend(var_list)
            self.prepare_data()
        else:
            for var in var_list:
                if var not in self.fields:
                    print "Variable [{:s}] is not recognized (it may be wrong)," \
                          "try to fetch from the server...".format(var)
                    success = self.add_field(var)
                    if not success:
                        return
        
        for var in var_list:
            if self._is_quarter_field(var):
                df_var = self.get_ts_quarter(var, start_date=self.extended_start_date_q)
            else:
                # must use extended date. Default is start_date
                df_var = self.get_ts(var, start_date=self.extended_start_date_d, end_date=self.end_date)
            
            var_df_dic[var] = df_var
        
        # TODO: send ann_date into expr.evaluate. We assume that ann_date of all fields of a symbol is the same
        df_ann = self._get_ann_df()
        if within_index:
            df_index_member = self.get_ts('index_member', start_date=self.extended_start_date_d, end_date=self.end_date)
            df_eval = parser.evaluate(var_df_dic, ann_dts=df_ann, trade_dts=self.dates, index_member=df_index_member)
        else:
            df_eval = parser.evaluate(var_df_dic, ann_dts=df_ann, trade_dts=self.dates)
        
        self.append_df(df_eval, field_name, is_quarterly=is_quarterly)
        
        if is_quarterly:
            df_ann = self._get_ann_df()
            df_expanded = align(df_eval, df_ann, self.dates)
            self.append_df(df_expanded, field_name, is_quarterly=False)
            
    def append_df(self, df, field_name, is_quarterly=False):
        """
        Append DataFrame to existing multi-index DataFrame and add corresponding field name.
        
        Parameters
        ----------
        df : pd.DataFrame or pd.Series
        field_name : str
        is_quarterly : bool
            Whether df is quarterly data (like quarterly financial statement) or daily data.

        """
        df = df.copy()
        if isinstance(df, pd.DataFrame):
            pass
        elif isinstance(df, pd.Series):
            df = pd.DataFrame(df)
        else:
            raise ValueError("Data to be appended must be pandas format. But we have {}".format(type(df)))
    
        if is_quarterly:
            the_data = self.data_q
        else:
            the_data = self.data_d
    
        exist_symbols = the_data.columns.levels[0]
        if len(df.columns) < len(exist_symbols):
            df2 = pd.DataFrame(index=df.index, columns=exist_symbols, data=np.nan)
            df2.update(df)
            df = df2
        multi_idx = pd.MultiIndex.from_product([exist_symbols, [field_name]])
        df.columns = multi_idx
    
        merge = the_data.join(df, how='left')  # left: keep index of existing data unchanged
        merge.sort_index(axis=1, level=['symbol', 'field'], inplace=True)
    
        if is_quarterly:
            self.data_q = merge
        else:
            self.data_d = merge
        self._add_field(field_name, is_quarterly)

    def remove_field(self, field_names):
        """
        Query and append new field to DataView.
        
        Parameters
        ----------
        field_names : str or list
            The (custom) field to be removed from dataview.
        
        Returns
        -------
        bool
            whether add successfully.

        """
        if isinstance(field_names, str):
            field_names = [field_names]
        elif isinstance(field_names, (list, tuple)):
            pass
        else:
            raise ValueError("field_names must be str or list of str.")
    
        for field_name in field_names:
            # parameter validation
            if field_name not in self.fields:
                print "Field name [{:s}] does not exist.".format(field_name)
                return
        
            if self._is_daily_field(field_name):
                is_quarterly = False
            elif self._is_quarter_field(field_name):
                is_quarterly = True
            else:
                print "Field name [{}] is a pre-defined field, ignore.".format(field_name)
                return
        
            # remove field data
            self.data_d = self.data_d.drop(field_name, axis=1, level=1)
            if is_quarterly:
                self.data_q = self.data_q.drop(field_name, axis=1, level=1)
        
            # remove fields name from list
            self.fields.remove(field_name)
            if is_quarterly:
                self.custom_quarterly_fields.remove(field_name)
            else:
                self.custom_daily_fields.remove(field_name)

    # --------------------------------------------------------------------------------------------------------
    # Get Data API
    def get(self, symbol="", start_date=0, end_date=0, fields=""):
        """
        Basic API to get arbitrary data. If nothing fetched, return None.
        
        Parameters
        ----------
        symbol : str, optional
            Separated by ',' default "" (all securities).
        start_date : int, optional
            Default 0 (self.start_date).
        end_date : int, optional
            Default 0 (self.start_date).
        fields : str, optional
            Separated by ',' default "" (all fields).

        Returns
        -------
        res : pd.DataFrame or None
            index is datetimeindex, columns are (symbol, fields) MultiIndex

        """
        sep = ','
    
        if not fields:
            fields = slice(None)  # self.fields
        else:
            fields = fields.split(sep)
    
        if not symbol:
            symbol = slice(None)  # this is 3X faster than symbol = self.symbol
        else:
            symbol = symbol.split(sep)
    
        if not start_date:
            start_date = self.start_date
        if not end_date:
            end_date = self.end_date
    
        res = self.data_d.loc[pd.IndexSlice[start_date: end_date], pd.IndexSlice[symbol, fields]]
    
        return res
    
    def get_snapshot(self, snapshot_date, symbol="", fields=""):
        """
        Get snapshot of given fields and symbol at snapshot_date.
        
        Parameters
        ----------
        snapshot_date : int
            Date of snapshot.
        symbol : str, optional
            Separated by ',' default "" (all securities).
        fields : str, optional
            Separated by ',' default "" (all fields).

        Returns
        -------
        res : pd.DataFrame
            symbol as index, field as columns

        """
        res = self.get(symbol=symbol, start_date=snapshot_date, end_date=snapshot_date, fields=fields)
        if res is None:
            print "No data. for date={}, fields={}, symbol={}".format(snapshot_date, fields, symbol)
            return
    
        res = res.stack(level='symbol', dropna=False)
        res.index = res.index.droplevel(level=self.TRADE_DATE_FIELD_NAME)
    
        return res
    
    def _get_ann_df(self):
        """
        Query announcement date of financial statements of all securities.
        
        Returns
        -------
        df_ann : pd.DataFrame or None
            Index is date, column is symbol.
            If no quarterly data available, return None.
        
        """
        if self.data_q is None:
            return None
        df_ann = self.data_q.loc[:, pd.IndexSlice[:, self.ANN_DATE_FIELD_NAME]]
        df_ann.columns = df_ann.columns.droplevel(level='field')
    
        return df_ann
    
    def get_ts_quarter(self, field, symbol="", start_date=0, end_date=0):
        # TODO
        sep = ','
        if not symbol:
            symbol = self.symbol
        else:
            symbol = symbol.split(sep)
    
        if not start_date:
            start_date = self.start_date
        if not end_date:
            end_date = self.end_date
    
        df_ref_quarterly = self.data_q.loc[:, pd.IndexSlice[symbol, field]]
        df_ref_quarterly.columns = df_ref_quarterly.columns.droplevel(level='field')
    
        return df_ref_quarterly
    
    def get_ts(self, field, symbol="", start_date=0, end_date=0):
        """
        Get time series data of single field.
        
        Parameters
        ----------
        field : str
            Single field.
        symbol : str, optional
            Separated by ',' default "" (all securities).
        start_date : int, optional
            Default 0 (self.start_date).
        end_date : int, optional
            Default 0 (self.start_date).

        Returns
        -------
        res : pd.DataFrame
            Index is int date, column is symbol.

        """
        res = self.get(symbol, start_date=start_date, end_date=end_date, fields=field)
        if res is None:
            print "No data. for start_date={}, end_date={}, field={}, symbol={}".format(start_date,
                                                                                        end_date, field, symbol)
            raise ValueError
            return
    
        res.columns = res.columns.droplevel(level='field')
    
        return res
    
    # --------------------------------------------------------------------------------------------------------
    # DataView I/O
    @staticmethod
    def _load_h5(fp):
        """Load data and meta_data from hd5 file.
        
        Parameters
        ----------
        fp : str, optional
            File path of pre-stored hd5 file.
        
        """
        h5 = pd.HDFStore(fp)
        
        res = dict()
        for key in h5.keys():
            res[key] = h5.get(key)
            
        h5.close()
        
        return res
        
    def load_dataview(self, folder_path='.'):
        """
        Load data from local file.
        
        Parameters
        ----------
        folder_path : str, optional
            Folder path to store hd5 file and meta data.
            
        """
        meta_data = jutil.read_json(os.path.join(folder_path, 'meta_data.json'))
        dic = self._load_h5(os.path.join(folder_path, 'data.hd5'))
        self.data_d = dic.get('/data_d', None)
        self.data_q = dic.get('/data_q', None)
        self._data_benchmark = dic.get('/data_benchmark', None)
        self._data_inst = dic.get('/data_inst', None)
        self.__dict__.update(meta_data)
        
        print "Dataview loaded successfully."

    def save_dataview(self, folder_path):
        """
        Save data and meta_data_to_store to a single hd5 file.
        Store at output/sub_folder
        
        Parameters
        ----------
        folder_path : str
            Path to store your data.

        """
        abs_folder = os.path.abspath(folder_path)
        meta_path = os.path.join(folder_path, 'meta_data.json')
        data_path = os.path.join(folder_path, 'data.hd5')
        
        data_to_store = {'data_d': self.data_d, 'data_q': self.data_q,
                         'data_benchmark': self.data_benchmark, 'data_inst': self.data_inst}
        data_to_store = {k: v for k, v in data_to_store.viewitems() if v is not None}
        meta_data_to_store = {key: self.__dict__[key] for key in self.meta_data_list}

        print "\nStore data..."
        jutil.save_json(meta_data_to_store, meta_path)
        self._save_h5(data_path, data_to_store)
        
        print ("Dataview has been successfully saved to:\n"
               + abs_folder + "\n\n"
               + "You can load it with load_dataview('{:s}')".format(abs_folder))

    @staticmethod
    def _save_h5(fp, dic):
        """
        Save data in dic to a hd5 file.
        
        Parameters
        ----------
        fp : str
            File path.
        dic : dict

        """
        import warnings
        warnings.filterwarnings('ignore', category=pd.io.pytables.PerformanceWarning)
        
        jutil.create_dir(fp)
        h5 = pd.HDFStore(fp, complevel=9, complib='blosc')
        for key, value in dic.viewitems():
            h5[key] = value
        h5.close()



# encoding: UTF-8

from jaqs.data import RemoteDataService
try:
    import pytest
except ImportError as e:
    if __name__ == "__main__":
        pass
    else:
        raise e
import jaqs.util as jutil

from config_path import DATA_CONFIG_PATH
data_config = jutil.read_json(DATA_CONFIG_PATH)


def test_remote_data_service_daily():
    # test daily
    res, msg = ds.daily('rb1710.SHF,600662.SH', fields="",
                        start_date=20170828, end_date=20170831,
                        adjust_mode=None)
    assert msg == '0,'
    
    rb = res.loc[res.loc[:, 'symbol'] == 'rb1710.SHF', :]
    stk = res.loc[res.loc[:, 'symbol'] == '600662.SH', :]
    assert set(rb.columns) == {'close', 'code', 'high', 'low', 'oi', 'open', 'settle', 'symbol', 'freq',
                               'trade_date', 'trade_status', 'turnover', 'volume', 'vwap'}
    assert rb.shape == (4, 14)
    assert rb.loc[:, 'volume'].values[0] == 189616
    assert stk.loc[:, 'volume'].values[0] == 7174813


def test_remote_data_service_daily_quited():
    # test daily
    res, msg = ds.daily('600832.SH', fields="",
                        start_date=20140828, end_date=20170831,
                        adjust_mode=None)
    assert msg == '0,'
    assert res.shape == (175, 14)


def test_remote_data_service_bar():
    # test bar
    res2, msg2 = ds.bar('rb1710.SHF,600662.SH', start_time=200000, end_time=160000, trade_date=20170831, fields="")
    assert msg2 == '0,'
    
    rb2 = res2.loc[res2.loc[:, 'symbol'] == 'rb1710.SHF', :]
    stk2 = res2.loc[res2.loc[:, 'symbol'] == '600662.SH', :]
    assert set(rb2.columns) == {u'close', u'code', u'date', u'freq', u'high', u'low', u'oi', u'open',
                                u'settle', u'symbol', u'time', u'trade_date', u'turnover', u'volume',
                                u'vwap'}
    assert abs(rb2.loc[:, 'settle'].values[0] - 0.0) < 1e-3
    assert rb2.shape == (345, 15)
    assert stk2.shape == (240, 15)
    assert rb2.loc[:, 'volume'].values[344] == 3366
    
    
def test_remote_data_service_lb():
    # test lb.secDailyIndicator
    fields = "pb,pe,free_share,net_assets,limit_status"
    for res3, msg3 in [ds.query("lb.secDailyIndicator", fields=fields,
                                filter="symbol=600030.SH&start_date=20170907&end_date=20170907",
                                orderby="trade_date"),
                       ds.query_lb_dailyindicator('600030.SH', 20170907, 20170907, fields)]:
        assert msg3 == '0,'
        assert abs(res3.loc[0, 'pb'] - 1.5135) < 1e-4
        assert abs(res3.loc[0, 'free_share'] - 781496.5954) < 1e-4
        assert abs(res3.loc[0, 'net_assets'] - 1.437e11) < 1e8
        assert res3.loc[0, 'limit_status'] == 0
    
    # test lb.income
    for res4, msg4 in [ds.query("lb.income", fields="",
                                filter="symbol=600000.SH&start_date=20150101&end_date=20170101&report_type=408001000",
                                order_by="report_date"),
                       ds.query_lb_fin_stat('income', '600000.SH', 20150101, 20170101, fields="")]:
        assert msg4 == '0,'
        assert res4.shape == (8, 12)
        assert abs(res4.loc[4, 'oper_rev'] - 120928000000) < 1


def test_remote_data_service_daily_ind_performance():
    hs300 = ds.get_index_comp('000300.SH', 20140101, 20170101)
    hs300_str = ','.join(hs300)
    
    fields = "pb,pe,share_float_free,net_assets,limit_status"
    res, msg = ds.query("lb.secDailyIndicator", fields=fields,
                          filter=("symbol=" + hs300_str
                                  + "&start_date=20160907&end_date=20170907"),
                          orderby="trade_date")
    assert msg == '0,'


def test_remote_data_service_components():
    res = ds.get_index_comp_df(index='000300.SH', start_date=20140101, end_date=20170505)
    assert res.shape == (814, 430)
    
    arr = ds.get_index_comp(index='000300.SH', start_date=20140101, end_date=20170505)
    assert len(arr) == 430


def test_remote_data_service_industry():
    from jaqs.data.align import align
    import pandas as pd
    
    arr = ds.get_index_comp(index='000300.SH', start_date=20130101, end_date=20170505)
    df = ds.get_industry_raw(symbol=','.join(arr), type_='ZZ')
    df = df.astype(dtype={'in_date': int})
    
    # df_ann = df.loc[:, ['in_date', 'symbol']]
    # df_ann = df_ann.set_index(['symbol', 'in_date'])
    # df_ann = df_ann.unstack(level='symbol')
    
    from jaqs.data import DataView
    dic_sec = jutil.group_df_to_dict(df, by='symbol')
    dic_sec = {sec: df.reset_index() for sec, df in dic_sec.viewitems()}
    
    df_ann = pd.concat([df.loc[:, 'in_date'].rename(sec) for sec, df in dic_sec.viewitems()], axis=1)
    df_value = pd.concat([df.loc[:, 'industry1_code'].rename(sec) for sec, df in dic_sec.viewitems()], axis=1)
    
    dates_arr = ds.get_trade_date_range(20140101, 20170505)
    res = align(df_value, df_ann, dates_arr)
    # df_ann = df.pivot(index='in_date', columns='symbol', values='in_date')
    # df_value = df.pivot(index=None, columns='symbol', values='industry1_code')
    
    def align_single_df(df_one_sec):
        df_value = df_one_sec.loc[:, ['industry1_code']]
        df_ann = df_one_sec.loc[:, ['in_date']]
        res = align(df_value, df_ann, dates_arr)
        return res
    # res_list = [align_single_df(df) for sec, df in dic_sec.viewitems()]
    res_list = [align_single_df(df) for df in dic_sec.values()[:10]]
    res = pd.concat(res_list, axis=1)
    
    
def test_remote_data_service_industry_df():
    # from jaqs.data import Calendar
    
    arr = ds.get_index_comp(index='000300.SH', start_date=20130101, end_date=20170505)
    symbol_arr = ','.join(arr)
    
    sec = '000008.SZ'
    type_ = 'ZZ'
    df_raw = ds.get_industry_raw(symbol=sec, type_=type_)
    df = ds.get_industry_daily(symbol=symbol_arr,
                               start_date=df_raw['in_date'].min(), end_date=20170505,
                               type_=type_, level=1)
    
    for idx, row in df_raw.iterrows():
        in_date = row['in_date']
        value = row['industry1_code']
        if in_date in df.index:
            assert df.loc[in_date, sec] == value
        else:
            idx = ds.get_next_trade_date(in_date)
            assert df.loc[idx, sec] == value
        

def test_remote_data_service_fin_indicator():
    symbol = '000008.SZ'
    filter_argument = ds._dic2url({'symbol': symbol})
    
    df_raw, msg = ds.query("lb.finIndicator", fields="",
                           filter=filter_argument, orderby="symbol")


def test_remote_data_service_adj_factor():
    arr = ds.get_index_comp(index='000300.SH', start_date=20130101, end_date=20170505)
    symbol_arr = ','.join(arr)
    
    res = ds.get_adj_factor_daily(symbol_arr, start_date=20130101, end_date=20170101, div=False)
    assert abs(res.loc[20160408, '300024.SZ'] - 10.735) < 1e-3
    assert abs(res.loc[20160412, '300024.SZ'] - 23.658) < 1e-3


def test_remote_data_service_inst_info():
    sec = '000001.SZ'
    res = ds.query_inst_info(sec, fields='status,selllot,buylot,pricetick,multiplier,product')
    assert res.at[sec, 'multiplier'] == 1
    assert abs(res.at[sec, 'pricetick'] - 0.01) < 1e-2
    assert res.at[sec, 'buylot'] == 100


def test_remote_data_service_index_weight():
    df = ds.get_index_weights(index='000300.SH', trade_date=20140101)
    assert df.shape[0] == 300
    assert abs(df['weight'].sum() - 1.0) < 1.0
    
    df = ds.get_index_weights(index='000016.SH', trade_date=20140101)
    assert df.shape[0] == 50
    assert abs(df['weight'].sum() - 1.0) < 1.0
    
    df = ds.get_index_weights_daily(index='000300.SH', start_date=20150101, end_date=20151221)
    assert abs(df.at[20150120, '000001.SZ'] - 1.07e-2) < 1e-2
    assert df.shape == (236, 321)


@pytest.fixture(autouse=True)
def my_globals(request):
    ds = RemoteDataService()
    ds.init_from_config(data_config)
    
    request.function.func_globals.update({'ds': ds})


if __name__ == "__main__":
    import time
    t_start = time.time()

    ds = RemoteDataService()
    ds.init_from_config(data_config)
    
    g = globals()
    g = {k: v for k, v in g.viewitems() if k.startswith('test_') and callable(v)}

    for test_name, test_func in g.viewitems():
        print "\nTesting {:s}...".format(test_name)
        test_func()
    print "Test Complete."
    
    t3 = time.time() - t_start
    print "\n\n\nTime lapsed in total: {:.1f}".format(t3)



# encoding: utf-8

import abc
from abc import abstractmethod
from six import with_metaclass

import numpy as np

from jaqs.data.basic import GoalPosition
from jaqs.util.sequence import SequenceGenerator
# import jaqs.util as jutil

from jaqs.trade import model
from jaqs.trade import common


class Strategy(with_metaclass(abc.ABCMeta)):
    """
    Abstract base class for strategies.

    Attributes
    ----------
    ctx : Context object
        Used to store relevant context of the strategy.
    run_mode : int
        Whether the trategy is under back-testing or live trading.
    pm : trade.PortfolioManger
        Responsible for managing orders, trades and positions.
    store : dict
        A dictionary to store variables that will be automatically saved.

    Methods
    -------

    """
    
    def __init__(self):
        super(Strategy, self).__init__()
        self.ctx = None
        # self.run_mode = common.RUN_MODE.BACKTEST
        
        # self.ctx.pm = PortfolioManager(strategy=self)
        # self.pm = self.ctx.pm

        # self.task_id_map = defaultdict(list)
        self.seq_gen = SequenceGenerator()

        self.init_balance = 0.0
        
    def init_from_config(self, props):
        pass
    
    def initialize(self):
        pass
    
    def _get_next_num(self, key):
        """used to generate id for orders and trades."""
        return str(np.int64(self.ctx.trade_date) * 10000 + self.seq_gen.get_next(key))
    
    '''
    # -------------------------------------------------------------------------------------------
    # Order

    def place_order(self, symbol, action, price, size, algo="", algo_param=None):
        """
        Send a request with an order to the system. Execution algorithm will be automatically chosen.
        Returns task_id which can be used to query execution and orders of this task.

        Parameters
        ----------
        symbol : str
            the symbol of symbol to be ordered, eg. "000001.SZ".
        action : str
        price : float.
            The price to be ordered at.
        size : int
            The quantity to be ordered at.
        algo : str, optional
            The algorithm to be used. If None then use default algorithm.
        algo_param : dict, optional
            Parameters of the algorithm. Default {}.

        Returns
        -------
        res : str
        msg : str.
            if res is None, message contains error information

        """
        pass

    def cancel_order(self, task_id):
        """Cancel all uncome orders of a task according to its task ID.

        Parameters
        ----------
        task_id : str
            ID of the task.
            NOTE we CANNOT cancel order by entrust_no because this may break the execution of algorithm.

        Returns
        -------
        result : str
            Indicate whether the cancel succeed.
        err_msg : str

        """
        pass

    # -------------------------------------------------------------------------------------------
    # Query

    def query_account(self):
        """
        
        Returns
        -------
        pd.DataFrame

        """
        pass

    def query_universe(self):
        """
        
        Returns
        -------
        pd.DataFrame

        """
        pass

    def query_position(self, mode="all", symbols=""):
        """
        Parameters
        ----------
        mode : str, optional
        symbols : str, optional
            Separated by ,
        
        Returns
        -------
        pd.DataFrame
        
        """
        pass

    def query_portfolio(self):
        """
        Return net positions of all securities in the strategy universe (including zero positions).

        Returns
        --------
        pd.DataFrame
            Current position of the strategy.

        """
        pass

    def query_task(self, task_id=-1):
        """
        Query order information of current day.

        Parameters
        ----------
        task_id : int, optional
            ID of the task. -1 by default (return all orders of the day; else return orders of this task).

        Returns
        -------
        pd.DataFrame

        """

    def query_order(self, task_id=-1):
        """
        Query order information of current day.

        Parameters
        ----------
        task_id : int
            ID of the task. -1 by default (return all orders of the day; else return orders of this task).

        Returns
        -------
        pd.DataFrame

        """
        pass

    def query_trade(self, task_id=-1):
        """
        Query trade information of current day.

        Parameters
        -----------
        task_id : int
            ID of the task. -1 by default (return all orders of the day; else return orders of this task).

        Returns
        --------
        pd.DataFrame

        """
        pass

    # -------------------------------------------------------------------------------------------
    # Portfolio Order

    def goal_portfolio(self, positions, algo="", algo_param=None):
        """
        Let the system automatically generate orders according to portfolio positions goal.
        If there are uncome orders of any symbol in the strategy universe, this order will be rejected. #TODO not impl

        Parameters
        -----------
        positions : list of GoalPosition
            This must include positions of all securities in the strategy universe.
            Use former value if there is no change.
        algo : str, optional
            The algorithm to be used. If None then use default algorithm.
        algo_param : dict, optional
            Parameters of the algorithm. Default {}.

        Returns
        --------
        result : bool
            Whether this command is accepted. True means the system's acceptance, instead of positions have changed.
        err_msg : str

        """
        pass

    def stop_portfolio(self):
        """
        Returns
        -------
        result : str
        message : str
            If result is None, message contains error information
        
        """
        pass

    def place_batch_order(self, orders, algo="", algo_param=None):
        """Send a batch of orders to the system together.

        Parameters
        -----------
        orders : list
            a list of trade.model.Order objects.
        algo : str, optional
            The algorithm to be used. If None then use default algorithm.
        algo_param : dict, optional
            Parameters of the algorithm. Default {}.

        Returns
        -------
        task_id : str
            Task ID generated by entrust_order.
        err_msg : str.

        """
        pass

    def basket_order(self, orders, algo="", algo_param=None):
        """
        Parameters
        ----------
        orders : list of dict
            [ {"security": "000001.SZ", "ref_price": 10.0, "inc_size" : 100}, ...]
        algo : str, optional
        algo_param : dict or None, optional
        
        Returns
        -------
        result : str
        message : str
            If result is None, message contains error information
        
        """
        pass
    '''
    
    # -------------------------------------------------------------------------------------------
    # Callback Indications & Responses
    
    def on_trade(self, ind):
        """

        Parameters
        ----------
        ind : TradeInd

        Returns
        -------

        """
        pass

    def on_order_status(self, ind):
        """

        Parameters
        ----------
        ind : OrderStatusInd

        Returns
        -------

        """
        pass
    
    def on_order_rsp(self, rsp):
        """
        
        Parameters
        ----------
        rsp

        """
        pass

    def on_task_rsp(self, rsp):
        """
        
        Parameters
        ----------
        rsp

        """
        pass

    def on_task_status(self, ind):
        """
        
        Parameters
        ----------
        rsp

        """
        pass


class AlphaStrategy(Strategy, model.FuncRegisterable):
    """
    Alpha strategy class.

    Attributes
    ----------
    period : str
        Interval between current and next. {'day', 'week', 'month'}
    days_delay : int
        n'th business day after next period.
    weights : np.array with the same shape with self.context.universe
    benchmark : str
        The benchmark symbol.
    risk_model : model.RiskModel
    signal_model : model.ReturnModel
    cost_model : model.CostModel

    Methods
    -------

    """
    # TODO register context
    def __init__(self, signal_model=None, stock_selector=None,
                 cost_model=None, risk_model=None,
                 pc_method="equal_weight"):
        super(AlphaStrategy, self).__init__()
        
        self.period = ""
        self.n_periods = 1
        self.days_delay = 0
        self.cash = 0
        self.position_ratio = 0.98
        
        self.risk_model = risk_model
        self.signal_model = signal_model
        self.cost_model = cost_model
        self.stock_selector = stock_selector
        
        self.weights = None
        
        self.pc_method = pc_method
        
        self.goal_positions = None

    def init_from_config(self, props):
        Strategy.init_from_config(self, props)
        
        self.cash = props.get('init_balance', 100000000)
        self.period = props.get('period', 'month')
        self.days_delay = props.get('days_delay', 0)
        self.n_periods = props.get('n_periods', 1)
        self.position_ratio = props.get('position_ratio', 0.98)

        self.use_pc_method(name='equal_weight', func=self.equal_weight, options=None)
        self.use_pc_method(name='mc', func=self.optimize_mc, options={'util_func': self.util_net_signal,
                                                                           'constraints': None,
                                                                           'initial_value': None})
        self.use_pc_method(name='factor_value_weight', func=self.factor_value_weight, options=None)
        self.use_pc_method(name='index_weight', func=self.index_weight, options=None)
        self.use_pc_method(name='market_value_weight', func=self.market_value_weight, options=None)
        
        self._validate_parameters()
        print "AlphaStrategy Initialized."
    
    def _validate_parameters(self):
        if self.pc_method in ['mc', 'quad_opt']:
            if self.signal_model is None and self.cost_model is None and self.risk_model is None:
                raise ValueError("At least one model of signal, cost and risk must be provided.")
        elif self.pc_method in ['factor_value_weight']:
            if self.signal_model is None:
                raise ValueError("signal_model must be provided when pc_method = 'factor_value_weight'")
        elif self.pc_method in ['equal_weight', 'index_weight', 'market_value_weight']:
            pass
        else:
            raise NotImplementedError("pc_method = {:s}".format(self.pc_method))
    
    def on_trade(self, ind):
        """

        Parameters
        ----------
        ind : TradeInd

        Returns
        -------

        """
        pass
        
    def use_pc_method(self, name, func, options=None):
        self._register_func(name, func, options)
    
    def _get_weights_last(self):
        current_positions = self.query_portfolio()
        univ_pos_dic = {p.symbol: p.current_size for p in current_positions}
        for sec in self.ctx.universe:
            if sec not in univ_pos_dic:
                univ_pos_dic[sec] = 0
        return univ_pos_dic

    def util_net_signal(self, weights_target):
        """
        util = net_signal = signal - all costs.
        
        Parameters
        ----------
        weights_target : dict
        
        """
        weights_last = self._get_weights_last()
    
        signal = self.signal_model.forecast_signal(weights_target)
        cost = self.cost_model.calc_cost(weights_last, weights_target)
        # liquid = self.liquid_model.calc_liquid(weight_now)
        risk = self.risk_model.calc_risk(weights_target)
    
        risk_coef = 1.0
        cost_coef = 1.0
        net_signal = signal - risk_coef * risk - cost_coef * cost  # - liquid * liq_factor
        return net_signal
    
    def portfolio_construction(self, universe_list=None):
        """
        Calculate target weights of each symbol in the strategy universe.
        User should not modify this function arbitrarily.
        
        Attributes
        ----------
        universe_list : list of str
            Symbols that should be considered during this re-balance.

        Returns
        -------
        self.weights : weights / GoalPosition (without rounding)
            Weights of each symbol.

        """
        # Step.1 filter and narrow down universe to sub-universe
        if self.stock_selector is not None:
            selected_list = self.stock_selector.get_selection()
            universe_list = [s for s in universe_list if s in selected_list]
        sub_univ = sorted(universe_list)
        
        self.ctx.snapshot_sub = self.ctx.snapshot.loc[sub_univ, :]
        
        # Step.2 pick the registered portfolio construction method
        rf = self.func_table[self.pc_method]
        func, options = rf.func, rf.options

        # Step.3 use the registered method to calculate weights and get weights for all symbols in universe
        weights_sub_universe, msg = func(**options)
        weights_all_universe = {symbol: weights_sub_universe.get(symbol, 0.0) for symbol in self.ctx.universe}
        if msg:
            print msg

        # if nan assign zero
        weights_all_universe = {k: 0.0 if np.isnan(v) else v for k, v in weights_all_universe.viewitems()}
        
        # normalize
        w_sum = np.sum(np.abs(weights_all_universe.values()))
        if w_sum > 1e-8:  # else all zeros weights
            weights_all_universe = {k: v / w_sum for k, v in weights_all_universe.viewitems()}

        self.weights = weights_all_universe

    def equal_weight(self):
        # discrete
        weights = {k: 1.0 for k in self.ctx.snapshot_sub.index.values}
        return weights, ''

    def market_value_weight(self):
        snap = self.ctx.snapshot_sub
        # TODO: pass options, instead of hard-code 'total_mv', 'float_mv'
        if 'total_mv' in snap.columns:
            mv = snap['total_mv']
        elif 'float_mv' in snap.columns:
            mv = snap['float_mv']
        else:
            raise ValueError("market_value_weight is chosen,"
                             "while no [float_mv] or [total_mv] field found in dataview.")
        mv = mv.fillna(0.0)
        weights = mv.to_dict()
        return weights, ""

    def index_weight(self):
        snap = self.ctx.snapshot_sub
        if 'index_weight' not in snap.columns:
            raise ValueError("index_weight is chosen,"
                             "while no [index_weight] field found in dataview.")
        ser_index_weight = snap['index_weight']
        ser_index_weight.fillna(0.0, inplace=True)
        weights = ser_index_weight.to_dict()
        return weights, ""
    
    def factor_value_weight(self):
        def long_only_weight_adjust(w):
            """
            Adjust weights for long only constraints.
            
            Parameters
            ----------
            w : dict
    
            Returns
            -------
            res : dict
    
            """
            # TODO: we should not add a const
            if not len(w):
                return w
            w_min = np.min(w.values())
            if w_min < 0:
                delta = 2 * abs(w_min)
            # if nan assign zero; else add const
                w = {k: v + delta for k, v in w.viewitems()}
            return w
        
        dic_forecasts = self.signal_model.make_forecast()
        weights = {k: 0.0 if (np.isnan(v) or np.isinf(v)) else v for k, v in dic_forecasts.viewitems()}
        weights = long_only_weight_adjust(weights)
        return weights, ""
        
    def optimize_mc(self, util_func):
        """
        Use naive search (Monte Carol) to find variable that maximize util_func.
        
        Parameters
        ----------
        util_func : callable
            Input variables, output the value of util function.

        Returns
        -------
        min_weights : dict
            best weights.
        msg : str
            error message.

        """
        n_exp = 5  # number of experiments of Monte Carol
        sub_univ = self.ctx.snapshot_sub.index.values
        n_var = len(sub_univ)
    
        weights_mat = np.random.rand(n_exp, n_var)
        weights_mat = weights_mat / weights_mat.sum(axis=1).reshape(-1, 1)
    
        min_f = 1e30
        min_weights = None
        for i in range(n_exp):
            weights = {sub_univ[j]: weights_mat[i, j] for j in range(n_var)}
            f = -util_func(weights)
            if f < min_f:
                min_weights = weights
                min_f = f
    
        if min_weights is None:
            msg = "No weights can make f > {:.2e} found in this search".format(min_f)
        else:
            msg = ""
        return min_weights, msg

    def re_weight_suspension(self, suspensions=None):
        """
        How we deal with weights when there are suspension securities.

        Parameters
        ----------
        suspensions : list of securities
            None if no suspension.

        """
        # TODO this can be refine: consider whether we increase or decrease shares on a suspended symbol.
        if not suspensions:
            return
        
        if len(suspensions) == len(self.ctx.universe):
            raise ValueError("All suspended")  # TODO custom error
        
        weights = {sec: w if sec not in suspensions else 0.0 for sec, w in self.weights.viewitems()}
        weights_sum = np.sum(np.abs(weights.values()))
        if weights_sum > 0.0:
            weights = {sec: w / weights_sum for sec, w in weights.viewitems()}
        
        self.weights = weights
    
    def on_after_rebalance(self, total):
        print "Before {} re-balance: available cash all (exclude suspensions) = {:9.4e}".format(self.ctx.trade_date, total)  # DEBUG
        pass
    
    def send_bullets(self):
        # self.ctx.trade_api.goal_portfolio_by_batch_order(self.goal_positions)
        self.ctx.trade_api.goal_portfolio(self.goal_positions, algo='vwap')
    
    def generate_weights_order(self, weights_dic, turnover, prices, suspensions=None):
        """
        Send order according subject to total turnover and weights of different securities.

        Parameters
        ----------
        weights_dic : dict of {symbol: weight}
            Weight of each symbol.
        turnover : float
            Total turnover goal of all securities. (cash quota)
        prices : dict of {str: float}
            {symbol: price}
        suspensions : list of str

        Returns
        -------
        goals : list of GoalPosition
        cash_left : float

        """
        # cash_left = 0.0
        cash_used = 0.0
        goals = []
        for sec, w in weights_dic.viewitems():
            goal_pos = dict()
            goal_pos['symbol'] = sec
            
            if sec in suspensions:
                current_pos = self.ctx.pm.get_position(sec)
                goal_pos['size'] = current_pos.current_size if current_pos is not None else 0
            elif abs(w) < 1e-8:
                # order.entrust_size = 0
                goal_pos['size'] = 0
            else:
                price = prices[sec]
                if not (np.isfinite(price) and np.isfinite(w)):
                    raise ValueError("NaN or Inf encountered! \n"
                                     "trade_date={}, symbol={}, price={}, weight={}".format(self.ctx.trade_date,
                                                                                            sec, price, w))
                shares_raw = w * turnover / price
                # shares unit 100
                shares = int(round(shares_raw / 100., 0)) * 100  # TODO cash may be not enough
                # shares_left = shares_raw - shares * 100  # may be negative
                # cash_left += shares_left * price
                cash_used += shares * price
                goal_pos['size'] = shares
            
            goals.append(goal_pos)
        
        cash_left = turnover - cash_used
        return goals, cash_left
    
    def query_portfolio(self):
        positions = []
        for sec in self.ctx.pm.holding_securities:
            positions.append(self.ctx.pm.get_position(sec))
        return positions


class EventDrivenStrategy(Strategy):
    def __init__(self):
        
        super(EventDrivenStrategy, self).__init__()
    
    @abstractmethod
    def on_quote(self, quote):
        pass
    
    def on_tick(self, quote):
        pass
    
    def on_cycle(self):
        pass
    
    def initialize(self):
        pass

    def cancel_all_orders(self):
        for task_id, task in self.ctx.pm.tasks.items():
            if task.trade_date == self.ctx.trade_date:
                if not task.is_finished:
                    self.ctx.trade_api.cancel_order(task_id)

    def liquidate(self, quote, n, tick_size=1.0, pos=0):
        self.cancel_all_orders()
        if pos == 0:
            return
    
        ref_price = quote.close
        if pos < 0:
            action = common.ORDER_ACTION.BUY
            price = ref_price + n * tick_size
        else:
            action = common.ORDER_ACTION.SELL
            price = ref_price - n * tick_size
        self.ctx.trade_api.place_order(quote.symbol, action, price, abs(pos))
